@@ -23,3 +23,18 @@ def list_zones(conn):
 
     for zone in conn.dns.zones():
         print(zone)
+
+
+def list_recordsets(conn, name_or_id):
+    print("List Recordsets for Zone")
+
+    zone = conn.dns.find_zone(name_or_id)
+
+    if zone:
+        zone_id = zone.id
+        recordsets = conn.dns.recordsets(zone_id)
+
+        for recordset in recordsets:
+            print(recordset)
+    else:
+        print("Zone not found.")
