@@ -20,9 +20,11 @@ from openstack.baremetal.v1 import chassis as _chassis
 from openstack.baremetal.v1 import conductor as _conductor
 from openstack.baremetal.v1 import deploy_templates as _deploytemplates
 from openstack.baremetal.v1 import driver as _driver
+from openstack.baremetal.v1 import inspection_rules as _inspectionrules
 from openstack.baremetal.v1 import node as _node
 from openstack.baremetal.v1 import port as _port
 from openstack.baremetal.v1 import port_group as _portgroup
+from openstack.baremetal.v1 import runbooks as _runbooks
 from openstack.baremetal.v1 import volume_connector as _volumeconnector
 from openstack.baremetal.v1 import volume_target as _volumetarget
 from openstack import exceptions
@@ -43,8 +45,10 @@ class Proxy(proxy.Proxy):
         "node": _node.Node,
         "port": _port.Port,
         "port_group": _portgroup.PortGroup,
+        "runbook": _runbooks.Runbook,
         "volume_connector": _volumeconnector.VolumeConnector,
         "volume_target": _volumetarget.VolumeTarget,
+        "inspection_rules": _inspectionrules.InspectionRule,
     }
 
     def _get_with_fields(self, resource_type, value, fields=None):
@@ -118,7 +122,9 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_chassis.Chassis, **attrs)
 
-    def find_chassis(self, name_or_id, ignore_missing=True):
+    # TODO(stephenfin): Delete this. You can't lookup a chassis by name so this
+    # is identical to get_chassis
+    def find_chassis(self, name_or_id, ignore_missing=True, *, details=True):
         """Find a single chassis.
 
         :param str name_or_id: The ID of a chassis.
@@ -126,11 +132,17 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.exceptions.NotFoundException` will be raised
             when the chassis does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent chassis.
+        :param details: A boolean indicating whether the detailed information
+            for the chassis should be returned.
+
         :returns: One :class:`~openstack.baremetal.v1.chassis.Chassis` object
             or None.
         """
         return self._find(
-            _chassis.Chassis, name_or_id, ignore_missing=ignore_missing
+            _chassis.Chassis,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            details=details,
         )
 
     def get_chassis(self, chassis, fields=None):
@@ -233,7 +245,7 @@ class Proxy(proxy.Proxy):
 
     def call_driver_vendor_passthru(
         self,
-        driver: ty.Union[str, _driver.Driver],
+        driver: str | _driver.Driver,
         verb: str,
         method: str,
         body: object = None,
@@ -319,7 +331,7 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_node.Node, **attrs)
 
-    def find_node(self, name_or_id, ignore_missing=True):
+    def find_node(self, name_or_id, ignore_missing=True, *, details=True):
         """Find a single node.
 
         :param str name_or_id: The name or ID of a node.
@@ -327,11 +339,16 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.exceptions.NotFoundException` will be raised
             when the node does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent node.
+        :param details: A boolean indicating whether the detailed information
+            for the node should be returned.
         :returns: One :class:`~openstack.baremetal.v1.node.Node` object
             or None.
         """
         return self._find(
-            _node.Node, name_or_id, ignore_missing=ignore_missing
+            _node.Node,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            details=details,
         )
 
     def get_node(self, node, fields=None):
@@ -889,7 +906,9 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_port.Port, **attrs)
 
-    def find_port(self, name_or_id, ignore_missing=True):
+    # TODO(stephenfin): Delete this. You can't lookup a port by name so this is
+    # identical to get_port
+    def find_port(self, name_or_id, ignore_missing=True, *, details=True):
         """Find a single port.
 
         :param str name_or_id: The ID of a port.
@@ -897,11 +916,16 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.exceptions.NotFoundException` will be raised
             when the port does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent port.
+        :param details: A boolean indicating whether the detailed information
+            for every port should be returned.
         :returns: One :class:`~openstack.baremetal.v1.port.Port` object
             or None.
         """
         return self._find(
-            _port.Port, name_or_id, ignore_missing=ignore_missing
+            _port.Port,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            details=details,
         )
 
     def get_port(self, port, fields=None):
@@ -1010,7 +1034,13 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_portgroup.PortGroup, **attrs)
 
-    def find_port_group(self, name_or_id, ignore_missing=True):
+    def find_port_group(
+        self,
+        name_or_id,
+        ignore_missing=True,
+        *,
+        details=True,
+    ):
         """Find a single port group.
 
         :param str name_or_id: The name or ID of a portgroup.
@@ -1018,11 +1048,16 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.exceptions.NotFoundException` will be raised
             when the port group does not exist.  When set to `True``, None will
             be returned when attempting to find a nonexistent port group.
+        :param details: A boolean indicating whether the detailed information
+            for the port group should be returned.
         :returns: One :class:`~openstack.baremetal.v1.port_group.PortGroup`
             object or None.
         """
         return self._find(
-            _portgroup.PortGroup, name_or_id, ignore_missing=ignore_missing
+            _portgroup.PortGroup,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            details=details,
         )
 
     def get_port_group(self, port_group, fields=None):
@@ -1087,16 +1122,65 @@ class Proxy(proxy.Proxy):
             _portgroup.PortGroup, port_group, ignore_missing=ignore_missing
         )
 
+    # ========== Virtual Media ==========
+
+    def attach_vmedia_to_node(
+        self,
+        node,
+        device_type,
+        image_url,
+        image_download_source=None,
+        retry_on_conflict=True,
+    ):
+        """Attach virtual media device to a node.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param device_type: The type of virtual media device.
+        :param image_url: The URL of the image to attach.
+        :param image_download_source: The source of the image download.
+        :param retry_on_conflict: Whether to retry HTTP CONFLICT errors.
+            This can happen when either the virtual media is already used on
+            a node or the node is locked. Since the latter happens more often,
+            the default value is True.
+        :return: ``None``
+        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
+            does not support the VMEDIA API.
+        """
+        res = self._get_resource(_node.Node, node)
+        res.attach_vmedia(
+            self,
+            device_type=device_type,
+            image_url=image_url,
+            image_download_source=image_download_source,
+            retry_on_conflict=retry_on_conflict,
+        )
+
+    def detach_vmedia_from_node(self, node, device_types=None):
+        """Detach virtual media from the node.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param device_types: A list with the types of virtual media
+            devices to detach.
+        :return: ``True`` if the virtual media was detached,
+            otherwise ``False``.
+        :raises: :exc:`~openstack.exceptions.NotSupported` if the server
+            does not support the VMEDIA API.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.detach_vmedia(self, device_types=device_types)
+
     # ========== VIFs ==========
 
     def attach_vif_to_node(
         self,
-        node: ty.Union[_node.Node, str],
+        node: _node.Node | str,
         vif_id: str,
         retry_on_conflict: bool = True,
         *,
-        port_id: ty.Optional[str] = None,
-        port_group_id: ty.Optional[str] = None,
+        port_id: str | None = None,
+        port_group_id: str | None = None,
     ) -> None:
         """Attach a VIF to the node.
 
@@ -1349,16 +1433,26 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_volumeconnector.VolumeConnector, **attrs)
 
-    def find_volume_connector(self, vc_id, ignore_missing=True):
+    # TODO(stephenfin): Delete this. You can't lookup a volume connector by
+    # name so this is identical to get_volume_connector
+    def find_volume_connector(
+        self,
+        vc_id,
+        ignore_missing=True,
+        *,
+        details=True,
+    ):
         """Find a single volume connector.
 
         :param str vc_id: The ID of a volume connector.
-
         :param bool ignore_missing: When set to ``False``, an exception of
             :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume connector does not exist.  When set to `True``,
             None will be returned when attempting to find a nonexistent
             volume connector.
+        :param details: A boolean indicating whether the detailed information
+            for the volume connector should be returned.
+
         :returns: One
             :class:`~openstack.baremetal.v1.volumeconnector.VolumeConnector`
             object or None.
@@ -1367,6 +1461,7 @@ class Proxy(proxy.Proxy):
             _volumeconnector.VolumeConnector,
             vc_id,
             ignore_missing=ignore_missing,
+            details=details,
         )
 
     def get_volume_connector(self, volume_connector, fields=None):
@@ -1498,22 +1593,29 @@ class Proxy(proxy.Proxy):
         """
         return self._create(_volumetarget.VolumeTarget, **attrs)
 
-    def find_volume_target(self, vt_id, ignore_missing=True):
+    # TODO(stephenfin): Delete this. You can't lookup a volume target by
+    # name so this is identical to get_volume_connector
+    def find_volume_target(self, vt_id, ignore_missing=True, *, details=True):
         """Find a single volume target.
 
         :param str vt_id: The ID of a volume target.
-
         :param bool ignore_missing: When set to ``False``, an exception of
             :class:`~openstack.exceptions.NotFoundException` will be raised
             when the volume connector does not exist.  When set to `True``,
             None will be returned when attempting to find a nonexistent
             volume target.
+        :param details: A boolean indicating whether the detailed information
+            for the volume target should be returned.
+
         :returns: One
             :class:`~openstack.baremetal.v1.volumetarget.VolumeTarget`
             object or None.
         """
         return self._find(
-            _volumetarget.VolumeTarget, vt_id, ignore_missing=ignore_missing
+            _volumetarget.VolumeTarget,
+            vt_id,
+            ignore_missing=ignore_missing,
+            details=details,
         )
 
     def get_volume_target(self, volume_target, fields=None):
@@ -1683,6 +1785,35 @@ class Proxy(proxy.Proxy):
             _deploytemplates.DeployTemplate, deploy_template, fields=fields
         )
 
+    def find_deploy_template(
+        self,
+        name_or_id,
+        ignore_missing=True,
+        *,
+        details=True,
+    ):
+        """Find a single deployment template.
+
+        :param str name_or_id: The name or ID of a deployment template.
+        :param bool ignore_missing: When set to ``False``, an exception of
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised
+            when the deployment template does not exist.  When set to `True``,
+            None will be returned when attempting to find a nonexistent
+            deployment template.
+        :param details: A boolean indicating whether the detailed information
+            for the deployment template should be returned.
+
+        :returns: One
+            :class:`~openstack.baremetal.v1.deploy_templates.DeployTemplate` or
+            None.
+        """
+        return self._find(
+            _deploytemplates.DeployTemplate,
+            name_or_id,
+            ignore_missing=ignore_missing,
+            details=details,
+        )
+
     def patch_deploy_template(self, deploy_template, patch):
         """Apply a JSON patch to the deploy_templates.
 
@@ -1701,6 +1832,106 @@ class Proxy(proxy.Proxy):
             _deploytemplates.DeployTemplate, deploy_template
         ).patch(self, patch)
 
+    # ========== Runbooks ==========
+
+    def runbooks(self, details=False, **query):
+        """Retrieve a generator of runbooks.
+
+        :param details: A boolean indicating whether the detailed information
+            for every runbook should be returned.
+        :param dict query: Optional query parameters to be sent to
+            restrict the runbooks to be returned.
+
+        :returns: A generator of Runbooks instances.
+        """
+        if details:
+            query['detail'] = True
+        return _runbooks.Runbook.list(self, **query)
+
+    def create_runbook(self, **attrs):
+        """Create a new runbook from attributes.
+
+        :param dict attrs: Keyword arguments that will be used to create a
+            :class:`~openstack.baremetal.v1.runbooks.Runbook`.
+
+        :returns: The results of runbook creation.
+        :rtype: :class:`~openstack.baremetal.v1.runbooks.Runbook`.
+        """
+        return self._create(_runbooks.Runbook, **attrs)
+
+    def update_runbook(self, runbook, **attrs):
+        """Update a runbook.
+
+        :param runbook: Either the ID of a runbook,
+            or an instance of
+            :class:`~openstack.baremetal.v1.runbooks.Runbook`.
+        :param dict attrs: The attributes to update on
+            the runbook represented
+            by the ``runbook`` parameter.
+
+        :returns: The updated runbook.
+        :rtype: :class:`~openstack.baremetal.v1.runbooks.Runbook`
+        """
+        return self._update(_runbooks.Runbook, runbook, **attrs)
+
+    def delete_runbook(self, runbook, ignore_missing=True):
+        """Delete a runbook.
+
+        :param runbook:The value can be
+            either the ID of a runbook or a
+            :class:`~openstack.baremetal.v1.runbooks.Runbook`
+            instance.
+
+        :param bool ignore_missing: When set to ``False``,
+            an exception:class:`~openstack.exceptions.NotFoundException`
+            will be raised when the runbook could not be found.
+            When set to ``True``, no exception will be raised when attempting
+            to delete a non-existent runbook.
+
+        :returns: The instance of the runbook which was deleted.
+        :rtype: :class:`~openstack.baremetal.v1.runbooks.Runbook`.
+        """
+
+        return self._delete(
+            _runbooks.Runbook,
+            runbook,
+            ignore_missing=ignore_missing,
+        )
+
+    def get_runbook(self, runbook, fields=None):
+        """Get a specific runbook.
+
+        :param runbook: The value can be the name or ID
+            of a runbook
+            :class:`~openstack.baremetal.v1.runbooks.Runbook`
+            instance.
+
+        :param fields: Limit the resource fields to fetch.
+
+        :returns: One
+            :class:`~openstack.baremetal.v1.runbooks.Runbook`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no runbook matching the name or ID could be found.
+        """
+        return self._get_with_fields(_runbooks.Runbook, runbook, fields=fields)
+
+    def patch_runbook(self, runbook, patch):
+        """Apply a JSON patch to the runbook.
+
+        :param runbooks: The value can be the ID of a
+            runbook or a :class:`~openstack.baremetal.v1.runbooks.Runbook`
+            instance.
+
+        :param patch: JSON patch to apply.
+
+        :returns: The updated runbook.
+        :rtype:
+            :class:`~openstack.baremetal.v1.runbooks.Runbook`
+        """
+        return self._get_resource(_runbooks.Runbook, runbook).patch(
+            self, patch
+        )
+
     # ========== Conductors ==========
 
     def conductors(self, details=False, **query):
@@ -1716,6 +1947,10 @@ class Proxy(proxy.Proxy):
             query['details'] = True
         return _conductor.Conductor.list(self, **query)
 
+    # NOTE(stephenfin): There is no 'find_conductor' since conductors are
+    # identified by the host name, not an arbitrary UUID, meaning
+    # 'find_conductor' would be identical to 'get_conductor'
+
     def get_conductor(self, conductor, fields=None):
         """Get a specific conductor.
 
@@ -1737,11 +1972,11 @@ class Proxy(proxy.Proxy):
         self,
         res: resource.ResourceT,
         status: str,
-        failures: ty.Optional[list[str]] = None,
-        interval: ty.Union[int, float, None] = 2,
-        wait: ty.Optional[int] = None,
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
         attribute: str = 'status',
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
+        callback: ty.Callable[[int], None] | None = None,
     ) -> resource.ResourceT:
         """Wait for the resource to be in a particular status.
 
@@ -1777,7 +2012,7 @@ class Proxy(proxy.Proxy):
         res: resource.ResourceT,
         interval: int = 2,
         wait: int = 120,
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
+        callback: ty.Callable[[int], None] | None = None,
     ) -> resource.ResourceT:
         """Wait for a resource to be deleted.
 
@@ -1793,3 +2028,105 @@ class Proxy(proxy.Proxy):
             to delete failed to occur in the specified seconds.
         """
         return resource.wait_for_delete(self, res, interval, wait, callback)
+
+    # ========== Inspection Rules ==========
+
+    def inspection_rules(self, details=False, **query):
+        """Retrieve a generator of inspection rules.
+
+        :param dict query: Optional query parameters to be sent to
+            restrict the inspection rules to be returned.
+
+        :returns: A generator of InspectionRule instances.
+        """
+        if details:
+            query['details'] = True
+        return _inspectionrules.InspectionRule.list(self, **query)
+
+    def create_inspection_rule(self, **attrs):
+        """Create a new inspection rule from attributes.
+
+        :param dict attrs: Keyword arguments that will be used to create a
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`.
+
+        :returns: The results of inspection rule creation.
+        :rtype:
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`.
+        """
+        return self._create(_inspectionrules.InspectionRule, **attrs)
+
+    def get_inspection_rule(self, inspection_rule, fields=None):
+        """Get a specific inspection rule.
+
+        :param inspection_rule: The ID of an inspection rule
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`
+            instance.
+
+        :param fields: Limit the resource fields to fetch.
+
+        :returns: One
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            inspection rule matching the ID could be found.
+        """
+        return self._get_with_fields(
+            _inspectionrules.InspectionRule, inspection_rule, fields=fields
+        )
+
+    def update_inspection_rule(self, inspection_rule, **attrs):
+        """Update an inspection rule.
+
+        :param inspection_rule: Either the ID of an inspection rule
+            or an instance of
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`.
+        :param dict attrs: The attributes to update on the
+            inspection rule represented by the ``inspection_rule`` parameter.
+
+        :returns: The updated inspection rule.
+        :rtype:
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`
+        """
+        return self._update(
+            _inspectionrules.InspectionRule, inspection_rule, **attrs
+        )
+
+    def delete_inspection_rule(self, inspection_rule, ignore_missing=True):
+        """Delete an inspection rule.
+
+        :param inspection_rule: The value can be either the ID of a
+            inspection_rule or a
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`
+            instance.
+        :param bool ignore_missing: When set to ``False``, an exception
+            :class:`~openstack.exceptions.NotFoundException` will be raised
+            when the inspection rule could not be found.
+            When set to ``True``, no exception will be raised when
+            attempting to delete a non-existent inspection rule.
+
+        :returns: The instance of the inspection rule which was deleted.
+        :rtype:
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`.
+        """
+        return self._delete(
+            _inspectionrules.InspectionRule,
+            inspection_rule,
+            ignore_missing=ignore_missing,
+        )
+
+    def patch_inspection_rule(self, inspection_rule, patch):
+        """Apply a JSON patch to the inspection rule.
+
+        :param inspection_rule: The value can be the ID of a
+            inspection_rule or a
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`
+            instance.
+
+        :param patch: JSON patch to apply.
+
+        :returns: The updated inspection rule.
+        :rtype:
+            :class:`~openstack.baremetal.v1.inspection_rules.InspectionRule`
+        """
+        return self._get_resource(
+            _inspectionrules.InspectionRule, inspection_rule
+        ).patch(self, patch)
