@@ -10,24 +10,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# import types so that we can reference ListType in sphinx param declarations.
-# We can't just use list, because sphinx gets confused by
-# openstack.resource.Resource.list and openstack.resource2.Resource.list
-
 from openstack.cloud import _utils
-from openstack.cloud import exc
+from openstack.cloud import openstackcloud
+from openstack import exceptions
 
 
-class CoeCloudMixin:
-
-    @_utils.cache_on_arguments()
+class CoeCloudMixin(openstackcloud._OpenStackCloudMixin):
     def list_coe_clusters(self):
         """List COE (Container Orchestration Engine) cluster.
 
         :returns: A list of container infrastructure management ``Cluster``
             objects.
-        :raises: ``OpenStackCloudException``: if something goes wrong during
-            the OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         return list(self.container_infrastructure_management.clusters())
 
@@ -41,8 +36,8 @@ class CoeCloudMixin:
 
         :returns: A list of container infrastructure management ``Cluster``
             objects.
-        :raises: ``OpenStackCloudException``: if something goes wrong during
-            the OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         coe_clusters = self.list_coe_clusters()
         return _utils._filter_list(coe_clusters, name_or_id, filters)
@@ -55,12 +50,7 @@ class CoeCloudMixin:
             A dictionary of meta data to use for further filtering. Elements
             of this dictionary may, themselves, be dictionaries. Example::
 
-                {
-                    'last_name': 'Smith',
-                    'other': {
-                        'gender': 'Female'
-                    }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
             A string containing a jmespath expression for further filtering.
@@ -72,7 +62,10 @@ class CoeCloudMixin:
         return _utils._get_entity(self, 'coe_cluster', name_or_id, filters)
 
     def create_coe_cluster(
-        self, name, cluster_template_id, **kwargs,
+        self,
+        name,
+        cluster_template_id,
+        **kwargs,
     ):
         """Create a COE cluster based on given cluster template.
 
@@ -80,11 +73,10 @@ class CoeCloudMixin:
         :param string cluster_template_id: ID of the cluster template to use.
         :param dict kwargs: Any other arguments to pass in.
 
-        :returns: a dict containing the cluster description
         :returns: The created container infrastructure management ``Cluster``
             object.
-        :raises: ``OpenStackCloudException`` if something goes wrong during
-            the OpenStack API call
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call
         """
         cluster = self.container_infrastructure_management.create_cluster(
             name=name,
@@ -92,17 +84,17 @@ class CoeCloudMixin:
             **kwargs,
         )
 
-        self.list_coe_clusters.invalidate(self)
         return cluster
 
     def delete_coe_cluster(self, name_or_id):
         """Delete a COE cluster.
 
         :param name_or_id: Name or unique ID of the cluster.
+
         :returns: True if the delete succeeded, False if the
             cluster was not found.
-
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
 
         cluster = self.get_coe_cluster(name_or_id)
@@ -116,7 +108,6 @@ class CoeCloudMixin:
             return False
 
         self.container_infrastructure_management.delete_cluster(cluster)
-        self.list_coe_clusters.invalidate(self)
         return True
 
     def update_coe_cluster(self, name_or_id, **kwargs):
@@ -127,17 +118,17 @@ class CoeCloudMixin:
 
         :returns: The updated cluster ``Cluster`` object.
 
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
-        self.list_coe_clusters.invalidate(self)
         cluster = self.get_coe_cluster(name_or_id)
         if not cluster:
-            raise exc.OpenStackCloudException(
-                "COE cluster %s not found." % name_or_id)
+            raise exceptions.SDKException(
+                f"COE cluster {name_or_id} not found."
+            )
 
         cluster = self.container_infrastructure_management.update_cluster(
-            cluster,
-            **kwargs
+            cluster, **kwargs
         )
 
         return cluster
@@ -149,8 +140,11 @@ class CoeCloudMixin:
 
         :returns: Details about the CA certificate for the given cluster.
         """
-        return self.container_infrastructure_management\
-            .get_cluster_certificate(cluster_id)
+        return (
+            self.container_infrastructure_management.get_cluster_certificate(
+                cluster_id
+            )
+        )
 
     def sign_coe_cluster_certificate(self, cluster_id, csr):
         """Sign client key and generate the CA certificate for a cluster
@@ -161,15 +155,13 @@ class CoeCloudMixin:
             certificate that client will use to communicate with the cluster.
 
         :returns: a dict representing the signed certs.
-
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
-        return self.container_infrastructure_management\
-            .create_cluster_certificate(
-                cluster_uuid=cluster_id,
-                csr=csr)
+        return self.container_infrastructure_management.create_cluster_certificate(  # noqa: E501
+            cluster_uuid=cluster_id, csr=csr
+        )
 
-    @_utils.cache_on_arguments()
     def list_cluster_templates(self, detail=False):
         """List cluster templates.
 
@@ -177,15 +169,16 @@ class CoeCloudMixin:
             ClusterTemplates are always returned with full details.
 
         :returns: a list of dicts containing the cluster template details.
-
-        :raises: ``OpenStackCloudException``: if something goes wrong during
-            the OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         return list(
-            self.container_infrastructure_management.cluster_templates())
+            self.container_infrastructure_management.cluster_templates()
+        )
 
     def search_cluster_templates(
-            self, name_or_id=None, filters=None, detail=False):
+        self, name_or_id=None, filters=None, detail=False
+    ):
         """Search cluster templates.
 
         :param name_or_id: cluster template name or ID.
@@ -194,13 +187,11 @@ class CoeCloudMixin:
             detailed output.
 
         :returns: a list of dict containing the cluster templates
-
-        :raises: ``OpenStackCloudException``: if something goes wrong during
-            the OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException`: if something goes
+            wrong during the OpenStack API call.
         """
         cluster_templates = self.list_cluster_templates(detail=detail)
-        return _utils._filter_list(
-            cluster_templates, name_or_id, filters)
+        return _utils._filter_list(cluster_templates, name_or_id, filters)
 
     def get_cluster_template(self, name_or_id, filters=None, detail=False):
         """Get a cluster template by name or ID.
@@ -210,12 +201,7 @@ class CoeCloudMixin:
             A dictionary of meta data to use for further filtering. Elements
             of this dictionary may, themselves, be dictionaries. Example::
 
-                {
-                    'last_name': 'Smith',
-                    'other': {
-                        'gender': 'Female'
-                    }
-                }
+                {'last_name': 'Smith', 'other': {'gender': 'Female'}}
 
             OR
             A string containing a jmespath expression for further filtering.
@@ -225,11 +211,16 @@ class CoeCloudMixin:
             cluster template is found.
         """
         return _utils._get_entity(
-            self, 'cluster_template', name_or_id,
-            filters=filters, detail=detail)
+            self,
+            'cluster_template',
+            name_or_id,
+            filters=filters,
+            detail=detail,
+        )
 
     def create_cluster_template(
-            self, name, image_id=None, keypair_id=None, coe=None, **kwargs):
+        self, name, image_id=None, keypair_id=None, coe=None, **kwargs
+    ):
         """Create a cluster template.
 
         :param string name: Name of the cluster template.
@@ -239,18 +230,18 @@ class CoeCloudMixin:
             Other arguments will be passed in kwargs.
 
         :returns: a dict containing the cluster template description
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during
-            the OpenStack API call
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call
         """
-        cluster_template = self.container_infrastructure_management \
-            .create_cluster_template(
+        cluster_template = (
+            self.container_infrastructure_management.create_cluster_template(
                 name=name,
                 image_id=image_id,
                 keypair_id=keypair_id,
                 coe=coe,
                 **kwargs,
             )
+        )
 
         return cluster_template
 
@@ -258,10 +249,11 @@ class CoeCloudMixin:
         """Delete a cluster template.
 
         :param name_or_id: Name or unique ID of the cluster template.
+
         :returns: True if the delete succeeded, False if the
             cluster template was not found.
-
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
 
         cluster_template = self.get_cluster_template(name_or_id)
@@ -270,11 +262,13 @@ class CoeCloudMixin:
             self.log.debug(
                 "Cluster template %(name_or_id)s does not exist",
                 {'name_or_id': name_or_id},
-                exc_info=True)
+                exc_info=True,
+            )
             return False
 
         self.container_infrastructure_management.delete_cluster_template(
-            cluster_template)
+            cluster_template
+        )
         return True
 
     def update_cluster_template(self, name_or_id, **kwargs):
@@ -283,26 +277,28 @@ class CoeCloudMixin:
         :param name_or_id: Name or ID of the cluster template being updated.
 
         :returns: an update cluster template.
-
-        :raises: OpenStackCloudException on operation error.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         cluster_template = self.get_cluster_template(name_or_id)
         if not cluster_template:
-            raise exc.OpenStackCloudException(
-                "Cluster template %s not found." % name_or_id)
-
-        cluster_template = self.container_infrastructure_management \
-            .update_cluster_template(
-                cluster_template,
-                **kwargs
+            raise exceptions.SDKException(
+                f"Cluster template {name_or_id} not found."
             )
 
+        cluster_template = (
+            self.container_infrastructure_management.update_cluster_template(
+                cluster_template, **kwargs
+            )
+        )
+
         return cluster_template
 
     def list_magnum_services(self):
         """List all Magnum services.
-        :returns: a list of dicts containing the service details.
 
-        :raises: OpenStackCloudException on operation error.
+        :returns: a list of dicts containing the service details.
+        :raises: :class:`~openstack.exceptions.SDKException` on operation
+            error.
         """
         return list(self.container_infrastructure_management.services())
