@@ -1276,7 +1276,7 @@ class NetworkCommonCloudMixin(openstackcloud._OpenStackCloudMixin):
             error.
         """
 
-        if type(ips) is list:
+        if type(ips) is not list:
             ips = [ips]
 
         for ip in ips:
@@ -1566,7 +1566,8 @@ class NetworkCommonCloudMixin(openstackcloud._OpenStackCloudMixin):
                 for address in port.get('fixed_ips', list()):
                     try:
                         ip = ipaddress.ip_address(address['ip_address'])
-                    except Exception:
+                    except Exception:  # noqa: S112
+                        # the address might be unset; ignore if so
                         continue
                     if ip.version == 4:
                         fixed_address = address['ip_address']
@@ -1729,7 +1730,6 @@ class NetworkCommonCloudMixin(openstackcloud._OpenStackCloudMixin):
         if not filters:
             filters = {}
 
-        data = []
         # Handle neutron security groups
         if self._use_neutron_secgroups():
             # pass filters dict to the list to filter as much as possible on
@@ -1815,7 +1815,6 @@ class NetworkCommonCloudMixin(openstackcloud._OpenStackCloudMixin):
                 "Unavailable feature: security groups"
             )
 
-        data = []
         security_group_json = {'name': name, 'description': description}
         if stateful is not None:
             security_group_json['stateful'] = stateful
