@@ -11,31 +11,12 @@
 # limitations under the License.
 
 from openstack.cloud import _utils
-from openstack.cloud import exc
+from openstack.cloud import openstackcloud
+from openstack import exceptions
 from openstack.orchestration.util import event_utils
-from openstack.orchestration.v1._proxy import Proxy
 
 
-def _no_pending_stacks(stacks):
-    """If there are any stacks not in a steady state, don't cache"""
-    for stack in stacks:
-        status = stack['stack_status']
-        if '_COMPLETE' not in status and '_FAILED' not in status:
-            return False
-    return True
-
-
-class OrchestrationCloudMixin:
-    orchestration: Proxy
-
-    # TODO(stephenfin): Remove final user of this
-    @property
-    def _orchestration_client(self):
-        if 'orchestration' not in self._raw_clients:
-            raw_client = self._get_raw_client('orchestration')
-            self._raw_clients['orchestration'] = raw_client
-        return self._raw_clients['orchestration']
-
+class OrchestrationCloudMixin(openstackcloud._OpenStackCloudMixin):
     def get_template_contents(
         self,
         template_file=None,
@@ -62,7 +43,7 @@ class OrchestrationCloudMixin:
         wait=False,
         timeout=3600,
         environment_files=None,
-        **parameters
+        **parameters,
     ):
         """Create a stack.
 
@@ -84,9 +65,8 @@ class OrchestrationCloudMixin:
         specified.
 
         :returns: a dict containing the stack description
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during
-            the OpenStack API call
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call
         """
         params = dict(
             tags=tags,
@@ -120,7 +100,7 @@ class OrchestrationCloudMixin:
         wait=False,
         timeout=3600,
         environment_files=None,
-        **parameters
+        **parameters,
     ):
         """Update a stack.
 
@@ -141,9 +121,8 @@ class OrchestrationCloudMixin:
         specified.
 
         :returns: a dict containing the stack description
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during
-            the OpenStack API calls
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API calls
         """
         params = dict(
             tags=tags,
@@ -183,9 +162,8 @@ class OrchestrationCloudMixin:
         :param boolean wait: Whether to wait for the delete to finish
 
         :returns: True if delete succeeded, False if the stack was not found.
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during
-            the OpenStack API call
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call
         """
         stack = self.get_stack(name_or_id, resolve_outputs=False)
         if stack is None:
@@ -206,11 +184,11 @@ class OrchestrationCloudMixin:
                 event_utils.poll_for_events(
                     self, stack_name=name_or_id, action='DELETE', marker=marker
                 )
-            except exc.OpenStackCloudHTTPError:
+            except exceptions.HttpException:
                 pass
             stack = self.get_stack(name_or_id, resolve_outputs=False)
             if stack and stack['stack_status'] == 'DELETE_FAILED':
-                raise exc.OpenStackCloudException(
+                raise exceptions.SDKException(
                     "Failed to delete stack {id}: {reason}".format(
                         id=name_or_id, reason=stack['stack_status_reason']
                     )
@@ -227,23 +205,21 @@ class OrchestrationCloudMixin:
 
         :returns: a list of ``openstack.orchestration.v1.stack.Stack``
             containing the stack description.
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during the
-            OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         stacks = self.list_stacks()
         return _utils._filter_list(stacks, name_or_id, filters)
 
-    @_utils.cache_on_arguments(should_cache_fn=_no_pending_stacks)
     def list_stacks(self, **query):
         """List all stacks.
 
         :param dict query: Query parameters to limit stacks.
+
         :returns: a list of :class:`openstack.orchestration.v1.stack.Stack`
             objects containing the stack description.
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during the
-            OpenStack API call.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call.
         """
         return list(self.orchestration.stacks(**query))
 
@@ -258,9 +234,9 @@ class OrchestrationCloudMixin:
 
         :returns: a :class:`openstack.orchestration.v1.stack.Stack`
             containing the stack description
-
-        :raises: ``OpenStackCloudException`` if something goes wrong during the
-            OpenStack API call or if multiple matches are found.
+        :raises: :class:`~openstack.exceptions.SDKException` if something goes
+            wrong during the OpenStack API call or if multiple matches are
+            found.
         """
 
         def _search_one_stack(name_or_id=None, filters=None):
@@ -274,7 +250,7 @@ class OrchestrationCloudMixin:
                 )
                 if stack.status == 'DELETE_COMPLETE':
                     return []
-            except exc.OpenStackCloudURINotFound:
+            except exceptions.NotFoundException:
                 return []
             return _utils._filter_list([stack], name_or_id, filters)
 
