@@ -39,9 +39,8 @@ def find_nova_interfaces(
                     # ext_tag is specified, but this interface has no tag
                     # We could actually return right away as this means that
                     # this cloud doesn't support OS-EXT-IPS. Nevertheless,
-                    # it would be better to perform an explicit check. e.g.:
-                    #   cloud._has_nova_extension('OS-EXT-IPS')
-                    # But this needs cloud to be passed to this function.
+                    # it would be better to perform an explicit check
+                    # but this needs cloud to be passed to this function.
                     continue
                 elif interface_spec['OS-EXT-IPS:type'] != ext_tag:
                     # Type doesn't match, continue with next one
@@ -52,9 +51,8 @@ def find_nova_interfaces(
                     # mac_addr is specified, but this interface has no mac_addr
                     # We could actually return right away as this means that
                     # this cloud doesn't support OS-EXT-IPS-MAC. Nevertheless,
-                    # it would be better to perform an explicit check. e.g.:
-                    #   cloud._has_nova_extension('OS-EXT-IPS-MAC')
-                    # But this needs cloud to be passed to this function.
+                    # it would be better to perform an explicit check
+                    # but this needs cloud to be passed to this function.
                     continue
                 elif interface_spec['OS-EXT-IPS-MAC:mac_addr'] != mac_addr:
                     # MAC doesn't match, continue with next one
@@ -222,7 +220,7 @@ def get_server_external_ipv4(cloud, server):
         for interface in interfaces:
             try:
                 ip = ipaddress.ip_address(interface['addr'])
-            except Exception:
+            except Exception:  # noqa: S112
                 # Skip any error, we're looking for a working ip - if the
                 # cloud returns garbage, it wouldn't be the first weird thing
                 # but it still doesn't meet the requirement of "be a working
@@ -247,7 +245,7 @@ def find_best_address(addresses, public=False, cloud_public=True):
         for address in addresses:
             try:
                 for count in utils.iterate_timeout(
-                    5, "Timeout waiting for %s" % address, wait=0.1
+                    5, f"Timeout waiting for {address}", wait=0.1
                 ):
                     # Return the first one that is reachable
                     try:
@@ -265,22 +263,23 @@ def find_best_address(addresses, public=False, cloud_public=True):
                             connect_socket.settimeout(1)
                             connect_socket.connect(sa)
                             return address
-                    except socket.error:
+                    except OSError:
                         # Sometimes a "no route to address" type error
                         # will fail fast, but can often come alive
                         # when retried.
                         continue
-            except Exception:
+            except Exception:  # noqa: S110
+                # This is best effort. Ignore any errors.
                 pass
 
     # Give up and return the first - none work as far as we can tell
     if do_check:
         log = _log.setup_logging('openstack')
         log.debug(
-            "The cloud returned multiple addresses %s:, and we could not "
+            f"The cloud returned multiple addresses {addresses}:, and we could not "
             "connect to port 22 on either. That might be what you wanted, "
             "but we have no clue what's going on, so we picked the first one "
-            "%s" % (addresses, addresses[0])
+            f"{addresses[0]}"
         )
     return addresses[0]
 
@@ -370,7 +369,7 @@ def get_groups_from_server(cloud, server, server_vars):
     groups.append(region)
 
     # And one by cloud_region
-    groups.append("%s_%s" % (cloud_name, region))
+    groups.append(f"{cloud_name}_{region}")
 
     # Check if group metadata key in servers' metadata
     group = server['metadata'].get('group')
@@ -381,21 +380,21 @@ def get_groups_from_server(cloud, server, server_vars):
         if extra_group:
             groups.append(extra_group)
 
-    groups.append('instance-%s' % server['id'])
+    groups.append('instance-{}'.format(server['id']))
 
     for key in ('flavor', 'image'):
         if 'name' in server_vars[key]:
-            groups.append('%s-%s' % (key, server_vars[key]['name']))
+            groups.append('{}-{}'.format(key, server_vars[key]['name']))
 
     for key, value in iter(server['metadata'].items()):
-        groups.append('meta-%s_%s' % (key, value))
+        groups.append(f'meta-{key}_{value}')
 
     az = server_vars.get('az', None)
     if az:
         # Make groups for az, region_az and cloud_region_az
         groups.append(az)
-        groups.append('%s_%s' % (region, az))
-        groups.append('%s_%s_%s' % (cloud.name, region, az))
+        groups.append(f'{region}_{az}')
+        groups.append(f'{cloud.name}_{region}_{az}')
     return groups
 
 
@@ -441,11 +440,11 @@ def _get_supplemental_addresses(cloud, server):
                     if fixed_net is None:
                         log = _log.setup_logging('openstack')
                         log.debug(
-                            "The cloud returned floating ip %(fip)s attached"
-                            " to server %(server)s but the fixed ip associated"
-                            " with the floating ip in the neutron listing"
-                            " does not exist in the nova listing. Something"
-                            " is exceptionally broken.",
+                            "The cloud returned floating ip %(fip)s attached "
+                            "to server %(server)s but the fixed ip associated "
+                            "with the floating ip in the neutron listing "
+                            "does not exist in the nova listing. Something "
+                            "is exceptionally broken.",
                             dict(fip=fip['id'], server=server['id']),
                         )
                     else:
