@@ -10,25 +10,34 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import typing as ty
+
 from openstack.dns.v2 import floating_ip as _fip
+from openstack.dns.v2 import limit as _limit
 from openstack.dns.v2 import recordset as _rs
 from openstack.dns.v2 import service_status as _svc_status
+from openstack.dns.v2 import tsigkey as _tsigkey
 from openstack.dns.v2 import zone as _zone
 from openstack.dns.v2 import zone_export as _zone_export
 from openstack.dns.v2 import zone_import as _zone_import
+from openstack.dns.v2 import zone_nameserver as _zone_nameserver
 from openstack.dns.v2 import zone_share as _zone_share
 from openstack.dns.v2 import zone_transfer as _zone_transfer
 from openstack import proxy
+from openstack import resource
 
 
 class Proxy(proxy.Proxy):
     _resource_registry = {
         "floating_ip": _fip.FloatingIP,
+        "limits": _limit.Limit,
         "recordset": _rs.Recordset,
         "service_status": _svc_status.ServiceStatus,
         "zone": _zone.Zone,
+        "tsigkey": _tsigkey.TSIGKey,
         "zone_export": _zone_export.ZoneExport,
         "zone_import": _zone_import.ZoneImport,
+        "zone_nameserver": _zone_nameserver.ZoneNameserver,
         "zone_share": _zone_share.ZoneShare,
         "zone_transfer_request": _zone_transfer.ZoneTransferRequest,
     }
@@ -61,6 +70,9 @@ class Proxy(proxy.Proxy):
         :returns: The results of zone creation.
         :rtype: :class:`~openstack.dns.v2.zone.Zone`
         """
+        if attrs.get('type') == "SECONDARY":
+            attrs.pop('email', None)
+            attrs.pop('ttl', None)
         return self._create(_zone.Zone, prepend_key=False, **attrs)
 
     def get_zone(self, zone):
@@ -147,6 +159,22 @@ class Proxy(proxy.Proxy):
         zone = self._get_resource(_zone.Zone, zone)
         return zone.xfr(self)
 
+    # ======== Zone nameservers ========
+    def zone_nameservers(self, zone):
+        """Retrieve a generator of nameservers for a zone
+
+        :param zone: The value can be the ID of a zone or a
+            :class:`~openstack.dns.v2.zone.Zone` instance.
+        :return: A generator of
+            :class:`~openstack.dns.v2.zone_nameserver.ZoneNameserver`
+            instances.
+        """
+        zone_id = resource.Resource._get_id(zone)
+        return self._list(
+            _zone_nameserver.ZoneNameserver,
+            zone_id=zone_id,
+        )
+
     # ======== Recordsets ========
     def recordsets(self, zone=None, **query):
         """Retrieve a generator of recordsets
@@ -658,6 +686,15 @@ class Proxy(proxy.Proxy):
             zone_id=zone_obj.id,
         )
 
+    # ======== Limits ========
+    def limits(self, **query):
+        """Retrieve a generator of limits
+
+        :returns: A generator of limits
+            (:class:`~openstack.dns.v2.limit.Limit`) instances
+        """
+        return self._list(_limit.Limit, **query)
+
     # ======== Service Statuses ========
     def service_statuses(self):
         """Retrieve a generator of service statuses
@@ -678,6 +715,68 @@ class Proxy(proxy.Proxy):
         """
         return self._get(_svc_status.ServiceStatus, service)
 
+    # ========== Utilities ==========
+    def wait_for_status(
+        self,
+        res: resource.ResourceT,
+        status: str,
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
+        attribute: str = 'status',
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for the resource to be in a particular status.
+
+        :param session: The session to use for making this request.
+        :param resource: The resource to wait on to reach the status. The
+            resource must have a status attribute specified via ``attribute``.
+        :param status: Desired status of the resource.
+        :param failures: Statuses that would indicate the transition
+            failed such as 'ERROR'. Defaults to ['ERROR'].
+        :param interval: Number of seconds to wait between checks.
+        :param wait: Maximum number of seconds to wait for transition.
+            Set to ``None`` to wait forever.
+        :param attribute: Name of the resource attribute that contains the
+            status.
+        :param callback: A callback function. This will be called with a single
+            value, progress. This is API specific but is generally a percentage
+            value from 0-100.
+
+        :return: The updated resource.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if the
+            transition to status failed to occur in ``wait`` seconds.
+        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
+            transitioned to one of the states in ``failures``.
+        :raises: :class:`~AttributeError` if the resource does not have a
+            ``status`` attribute
+        """
+        return resource.wait_for_status(
+            self, res, status, failures, interval, wait, attribute, callback
+        )
+
+    def wait_for_delete(
+        self,
+        res: resource.ResourceT,
+        interval: int = 2,
+        wait: int = 120,
+        callback: ty.Callable[[int], None] | None = None,
+    ) -> resource.ResourceT:
+        """Wait for a resource to be deleted.
+
+        :param res: The resource to wait on to be deleted.
+        :param interval: Number of seconds to wait before to consecutive
+            checks.
+        :param wait: Maximum number of seconds to wait before the change.
+        :param callback: A callback function. This will be called with a single
+            value, progress, which is a percentage value from 0-100.
+
+        :returns: The resource is returned on success.
+        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
+            to delete failed to occur in the specified seconds.
+        """
+        return resource.wait_for_delete(self, res, interval, wait, callback)
+
     def _get_cleanup_dependencies(self):
         # DNS may depend on floating ip
         return {'dns': {'before': ['network']}}
@@ -719,3 +818,76 @@ class Proxy(proxy.Proxy):
                     filters=filters,
                     resource_evaluation_fn=resource_evaluation_fn,
                 )
+
+    # ====== TSIG keys ======
+    def tsigkeys(self, **query):
+        """Retrieve a generator of zones
+
+        :param dict query: Optional query parameters to be sent to limit the
+            resources being returned.
+
+        :returns: A generator of zone
+            :class: `~openstack.dns.v2.tsigkey.TSIGKey` instances.
+        """
+        return self._list(_tsigkey.TSIGKey, **query)
+
+    def create_tsigkey(self, **attrs):
+        """Create a new tsigkey from attributes
+
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.dns.v2.tsigkey.Tsigkey`,
+            comprised of the properties on the Tsigkey class.
+        :returns: The results of zone creation.
+        :rtype: :class:`~openstack.dns.v2.tsigkey.Tsigkey`
+        """
+        return self._create(_tsigkey.TSIGKey, prepend_key=False, **attrs)
+
+    def get_tsigkey(self, tsigkey):
+        """Get a zone
+
+        :param tsigkey: The value can be the ID of a tsigkey
+            or a :class:'~openstack.dns.v2.tsigkey.TSIGKey' instance.
+        :returns: A generator of tsigkey
+            :class:'~openstack.dns.v2.tsigkey.TSIGKey' instances.
+        """
+        return self._get(_tsigkey.TSIGKey, tsigkey)
+
+    def delete_tsigkey(
+        self, tsigkey, ignore_missing=True, delete_shares=False
+    ):
+        """Delete a TSIG key
+
+        :param tsigkey: The value can be the ID of a TSIG key
+            or a :class:`~openstack.dns.v2.tsigkey.TSIGKey` instance.
+        :param bool ignore_missing: When set to ``False``
+            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            the TSIG key does not exist.
+            When set to ``True``, no exception will be set when attempting to
+            delete a nonexistent TSIG key.
+
+        :returns: TSIG Key that has been deleted
+        :rtype: :class:`~openstack.dns.v2.tsigkey.TSIGKey`
+        """
+
+        return self._delete(
+            _tsigkey.TSIGKey,
+            tsigkey,
+            ignore_missing=ignore_missing,
+            delete_shares=delete_shares,
+        )
+
+    def find_tsigkey(self, name_or_id, ignore_missing=True):
+        """Find a single tsigkey
+
+        :param name_or_id: The name or ID of a tsigkey
+        :param bool ignore_missing: When set to ``False``
+            :class: `!openstack.exceptions.ResourceNotFound` will be raised
+            when the tsigkey does not exit.
+            Wehn set to ``True``, no exception will be set when attempting
+            to delete a nonexitstent zone.
+
+        :returns::class:`~openstack.dns.v2.tsigkey.TSIGKey`
+        """
+        return self._find(
+            _tsigkey.TSIGKey, name_or_id, ignore_missing=ignore_missing
+        )
