@@ -51,7 +51,7 @@ class ZoneImport(_base.Resource):
     #: ID for the zone that was created by this import
     zone_id = resource.Body('zone_id')
 
-    def create(self, session, prepend_key=True, base_path=None):
+    def create(self, session, prepend_key=True, base_path=None, **kwargs):
         """Create a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -70,7 +70,7 @@ class ZoneImport(_base.Resource):
             raise exceptions.MethodNotSupported(self, "create")
 
         session = self._get_session(session)
-        microversion = self._get_microversion(session, action='create')
+        microversion = self._get_microversion(session)
         # Create ZoneImport requires empty body and 'text/dns' as content-type
         # skip _prepare_request completely, since we need just empty body
         request = resource._Request(
