@@ -9,13 +9,17 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+
+import typing as ty
+
+from openstack.common import tag
 from openstack import exceptions
 from openstack.image import _download
 from openstack import resource
 from openstack import utils
 
 
-class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
+class Image(resource.Resource, tag.TagMixin, _download.DownloadMixin):
     resources_key = 'images'
     base_path = '/images'
 
@@ -33,13 +37,24 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     _store_unknown_attrs_as_properties = True
 
     _query_mapping = resource.QueryParameters(
-        "name", "visibility",
-        "member_status", "owner",
-        "status", "size_min", "size_max",
-        "protected", "is_hidden",
-        "sort_key", "sort_dir", "sort", "tag",
-        "created_at", "updated_at",
-        is_hidden="os_hidden")
+        "id",
+        "name",
+        "visibility",
+        "member_status",
+        "owner",
+        "status",
+        "size_min",
+        "size_max",
+        "protected",
+        "is_hidden",
+        "sort_key",
+        "sort_dir",
+        "sort",
+        "tag",
+        "created_at",
+        "updated_at",
+        is_hidden="os_hidden",
+    )
 
     # NOTE: Do not add "self" support here. If you've used Python before,
     # you know that self, while not being a reserved word, has special
@@ -138,7 +153,9 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     #: Optional property allows created servers to have a different bandwidth
     #: cap than that defined in the network they are attached to.
     instance_type_rxtx_factor = resource.Body(
-        "instance_type_rxtx_factor", type=float)
+        "instance_type_rxtx_factor",
+        type=float,
+    )
     # For snapshot images, this is the UUID of the server used to
     #: create this image.
     instance_uuid = resource.Body('instance_uuid')
@@ -209,7 +226,9 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     #: number of guest vCPUs. This makes the network performance scale
     #: across a number of vCPUs.
     is_hw_vif_multiqueue_enabled = resource.Body(
-        'hw_vif_multiqueue_enabled', type=bool)
+        'hw_vif_multiqueue_enabled',
+        type=bool,
+    )
     #: If true, enables the BIOS bootmenu.
     is_hw_boot_menu_enabled = resource.Body('hw_boot_menu', type=bool)
     #: The virtual SCSI or IDE controller used by the hypervisor.
@@ -235,7 +254,7 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     def _action(self, session, action):
         """Call an action on an image ID."""
         url = utils.urljoin(self.base_path, self.id, 'actions', action)
-        return session.post(url,)
+        return session.post(url)
 
     def deactivate(self, session):
         """Deactivate an image
@@ -252,61 +271,103 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
         """
         self._action(session, "reactivate")
 
-    def upload(self, session):
-        """Upload data into an existing image"""
+    def upload(self, session, *, data=None):
+        """Upload data into an existing image
+
+        :param session: The session to use for making this request
+        :param data: Optional data to be uploaded. If not provided, the
+            `~Image.data` attribute will be used
+        :returns: The server response
+        """
+        if data:
+            self.data = data
         url = utils.urljoin(self.base_path, self.id, 'file')
-        return session.put(url, data=self.data,
-                           headers={"Content-Type": "application/octet-stream",
-                                    "Accept": ""})
+        return session.put(
+            url,
+            data=self.data,
+            headers={"Content-Type": "application/octet-stream", "Accept": ""},
+        )
+
+    def stage(self, session, *, data=None):
+        """Stage binary image data into an existing image
+
+        :param session: The session to use for making this request
+        :param data: Optional data to be uploaded. If not provided, the
+            `~Image.data` attribute will be used
+        :returns: The server response
+        """
+        if data:
+            self.data = data
 
-    def stage(self, session):
-        """Stage binary image data into an existing image"""
         url = utils.urljoin(self.base_path, self.id, 'stage')
         response = session.put(
-            url, data=self.data,
-            headers={"Content-Type": "application/octet-stream",
-                     "Accept": ""})
+            url,
+            data=self.data,
+            headers={"Content-Type": "application/octet-stream", "Accept": ""},
+        )
         self._translate_response(response, has_body=False)
         return self
 
-    def import_image(self, session, method='glance-direct', uri=None,
-                     store=None, stores=None, all_stores=None,
-                     all_stores_must_succeed=None):
+    def import_image(
+        self,
+        session,
+        method='glance-direct',
+        *,
+        uri=None,
+        remote_region=None,
+        remote_image_id=None,
+        remote_service_interface=None,
+        store=None,
+        stores=None,
+        all_stores=None,
+        all_stores_must_succeed=None,
+    ):
         """Import Image via interoperable image import process"""
         if all_stores and (store or stores):
             raise exceptions.InvalidRequest(
-                "all_stores is mutually exclusive with"
-                " store and stores")
+                'all_stores is mutually exclusive with store and stores'
+            )
         if store and stores:
             raise exceptions.InvalidRequest(
-                "store and stores are mutually exclusive."
-                " Please just use stores.")
+                'store and stores are mutually exclusive. stores should be '
+                'preferred.'
+            )
         if store:
             stores = [store]
         else:
             stores = stores or []
 
         url = utils.urljoin(self.base_path, self.id, 'import')
-        json = {'method': {'name': method}}
+        data: dict[str, ty.Any] = {'method': {'name': method}}
+
         if uri:
-            if method == 'web-download':
-                json['method']['uri'] = uri
-            else:
-                raise exceptions.InvalidRequest('URI is only supported with '
-                                                'method: "web-download"')
+            if method != 'web-download':
+                raise exceptions.InvalidRequest(
+                    'URI is only supported with method: "web-download"'
+                )
+            data['method']['uri'] = uri
+
+        if remote_region and remote_image_id:
+            if remote_service_interface:
+                data['method']['glance_service_interface'] = (
+                    remote_service_interface
+                )
+                data['method']['glance_region'] = remote_region
+                data['method']['glance_image_id'] = remote_image_id
+
         if all_stores is not None:
-            json['all_stores'] = all_stores
+            data['all_stores'] = all_stores
         if all_stores_must_succeed is not None:
-            json['all_stores_must_succeed'] = all_stores_must_succeed
-        for s in stores:
-            json.setdefault('stores', [])
-            json['stores'].append(s.id)
+            data['all_stores_must_succeed'] = all_stores_must_succeed
+        if stores:
+            data['stores'] = [s.id for s in stores]
 
         headers = {}
         # Backward compat
         if store is not None:
             headers = {'X-Image-Meta-Store': store.id}
-        session.post(url, json=json, headers=headers)
+
+        return session.post(url, json=data, headers=headers)
 
     def _consume_header_attrs(self, attrs):
         self.image_import_methods = []
@@ -316,16 +377,27 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
 
         return super()._consume_header_attrs(attrs)
 
-    def _prepare_request(self, requires_id=None, prepend_key=False,
-                         patch=False, base_path=None, **kwargs):
-        request = super(Image, self)._prepare_request(requires_id=requires_id,
-                                                      prepend_key=prepend_key,
-                                                      patch=patch,
-                                                      base_path=base_path)
+    def _prepare_request(
+        self,
+        requires_id=None,
+        prepend_key=False,
+        patch=False,
+        base_path=None,
+        params=None,
+        **kwargs,
+    ):
+        request = super()._prepare_request(
+            requires_id=requires_id,
+            prepend_key=prepend_key,
+            patch=patch,
+            base_path=base_path,
+            params=params,
+            **kwargs,
+        )
         if patch:
             headers = {
                 'Content-Type': 'application/openstack-images-v2.1-json-patch',
-                'Accept': ''
+                'Accept': '',
             }
             request.headers.update(headers)
 
@@ -334,8 +406,7 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
     @classmethod
     def find(cls, session, name_or_id, ignore_missing=True, **params):
         # Do a regular search first (ignoring missing)
-        result = super(Image, cls).find(session, name_or_id, True,
-                                        **params)
+        result = super().find(session, name_or_id, True, **params)
 
         if result:
             return result
@@ -350,5 +421,6 @@ class Image(resource.Resource, resource.TagMixin, _download.DownloadMixin):
 
         if ignore_missing:
             return None
-        raise exceptions.ResourceNotFound(
-            "No %s found for %s" % (cls.__name__, name_or_id))
+        raise exceptions.NotFoundException(
+            f"No {cls.__name__} found for {name_or_id}"
+        )
