@@ -1044,7 +1044,7 @@ class Proxy(proxy.Proxy):
                 method.upper(),
             )
 
-        expiration: ty.Union[float, int]
+        expiration: float | int
         if not absolute:
             expiration = _get_expiration(timestamp)
         else:
@@ -1129,11 +1129,11 @@ class Proxy(proxy.Proxy):
         self,
         res: resource.ResourceT,
         status: str,
-        failures: ty.Optional[list[str]] = None,
-        interval: ty.Union[int, float, None] = 2,
-        wait: ty.Optional[int] = None,
+        failures: list[str] | None = None,
+        interval: int | float | None = 2,
+        wait: int | None = None,
         attribute: str = 'status',
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
+        callback: ty.Callable[[int], None] | None = None,
     ) -> resource.ResourceT:
         """Wait for the resource to be in a particular status.
 
@@ -1169,7 +1169,7 @@ class Proxy(proxy.Proxy):
         res: resource.ResourceT,
         interval: int = 2,
         wait: int = 120,
-        callback: ty.Optional[ty.Callable[[int], None]] = None,
+        callback: ty.Callable[[int], None] | None = None,
     ) -> resource.ResourceT:
         """Wait for a resource to be deleted.
 
