@@ -29,11 +29,13 @@ class StackFiles(resource.Resource):
     # Backwards compat
     stack_name = name
     #: ID of the stack where the template is referenced.
-    id = resource.URI('stack_id')  # type: ignore
+    id = resource.URI('stack_id')
     # Backwards compat
-    stack_id = id  # type: ignore
+    stack_id = id
 
-    def fetch(self, session, base_path=None):
+    def fetch(
+        self, session, requires_id=False, base_path=None, *args, **kwargs
+    ):
         # The stack files response contains a map of filenames and file
         # contents.
         request = self._prepare_request(requires_id=False, base_path=base_path)
