@@ -31,7 +31,6 @@ _TRUE_VALUES = ('true', '1', 'yes')
 
 
 class TestCase(base.BaseTestCase):
-
     """Test case base class for all tests."""
 
     # A way to adjust slow test classes
@@ -97,9 +96,7 @@ class TestCase(base.BaseTestCase):
             first = first.toDict()
         if isinstance(second, utils.Munch):
             second = second.toDict()
-        return super(TestCase, self).assertEqual(
-            first, second, *args, **kwargs
-        )
+        return super().assertEqual(first, second, *args, **kwargs)
 
     def printLogs(self, *args):
         self._log_stream.seek(0)
@@ -135,9 +132,7 @@ class TestCase(base.BaseTestCase):
             if not whole[key] and part[key]:
                 missing_keys.append(key)
         if missing_keys:
-            self.fail(
-                "Keys %s are in %s but not in %s" % (missing_keys, part, whole)
-            )
+            self.fail(f"Keys {missing_keys} are in {part} but not in {whole}")
         wrong_values = [
             (key, part[key], whole[key])
             for key in part
@@ -145,8 +140,10 @@ class TestCase(base.BaseTestCase):
         ]
         if wrong_values:
             self.fail(
-                "Mismatched values: %s"
-                % ", ".join(
-                    "for %s got %s and %s" % tpl for tpl in wrong_values
+                "Mismatched values: {}".format(
+                    ", ".join(
+                        "for {} got {} and {}".format(*tpl)
+                        for tpl in wrong_values
+                    )
                 )
             )
