@@ -24,7 +24,7 @@ from openstack.tests.functional import base
 
 class TestQosBandwidthLimitRule(base.BaseFunctionalTest):
     def setUp(self):
-        super(TestQosBandwidthLimitRule, self).setUp()
+        super().setUp()
         if not self.operator_cloud:
             self.skipTest("Operator cloud is required for this test")
         if not self.operator_cloud.has_service('network'):
@@ -41,7 +41,7 @@ class TestQosBandwidthLimitRule(base.BaseFunctionalTest):
         try:
             self.operator_cloud.delete_qos_policy(self.policy['id'])
         except Exception as e:
-            raise exceptions.SDKException(e)
+            raise exceptions.SDKException(str(e))
 
     def test_qos_bandwidth_limit_rule_lifecycle(self):
         max_kbps = 1500
