@@ -15,7 +15,9 @@ from unittest import mock
 from openstack.baremetal.v1 import _proxy
 from openstack.baremetal.v1 import allocation
 from openstack.baremetal.v1 import chassis
+from openstack.baremetal.v1 import deploy_templates
 from openstack.baremetal.v1 import driver
+from openstack.baremetal.v1 import inspection_rules
 from openstack.baremetal.v1 import node
 from openstack.baremetal.v1 import port
 from openstack.baremetal.v1 import port_group
@@ -31,7 +33,7 @@ _MOCK_METHOD = 'openstack.baremetal.v1._proxy.Proxy._get_with_fields'
 
 class TestBaremetalProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestBaremetalProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -60,7 +62,11 @@ class TestChassis(TestBaremetalProxy):
         self.verify_create(self.proxy.create_chassis, chassis.Chassis)
 
     def test_find_chassis(self):
-        self.verify_find(self.proxy.find_chassis, chassis.Chassis)
+        self.verify_find(
+            self.proxy.find_chassis,
+            chassis.Chassis,
+            expected_kwargs={'details': True},
+        )
 
     def test_get_chassis(self):
         self.verify_get(
@@ -110,7 +116,11 @@ class TestNode(TestBaremetalProxy):
         self.verify_create(self.proxy.create_node, node.Node)
 
     def test_find_node(self):
-        self.verify_find(self.proxy.find_node, node.Node)
+        self.verify_find(
+            self.proxy.find_node,
+            node.Node,
+            expected_kwargs={'details': True},
+        )
 
     def test_get_node(self):
         self.verify_get(
@@ -162,7 +172,11 @@ class TestPort(TestBaremetalProxy):
         self.verify_create(self.proxy.create_port, port.Port)
 
     def test_find_port(self):
-        self.verify_find(self.proxy.find_port, port.Port)
+        self.verify_find(
+            self.proxy.find_port,
+            port.Port,
+            expected_kwargs={'details': True},
+        )
 
     def test_get_port(self):
         self.verify_get(
@@ -236,7 +250,9 @@ class TestVolumeConnector(TestBaremetalProxy):
 
     def test_find_volume_connector(self):
         self.verify_find(
-            self.proxy.find_volume_connector, volume_connector.VolumeConnector
+            self.proxy.find_volume_connector,
+            volume_connector.VolumeConnector,
+            expected_kwargs={'details': True},
         )
 
     def test_get_volume_connector(self):
@@ -282,7 +298,9 @@ class TestVolumeTarget(TestBaremetalProxy):
 
     def test_find_volume_target(self):
         self.verify_find(
-            self.proxy.find_volume_target, volume_target.VolumeTarget
+            self.proxy.find_volume_target,
+            volume_target.VolumeTarget,
+            expected_kwargs={'details': True},
         )
 
     def test_get_volume_target(self):
@@ -304,6 +322,55 @@ class TestVolumeTarget(TestBaremetalProxy):
         )
 
 
+class TestDeployTemplate(TestBaremetalProxy):
+    @mock.patch.object(deploy_templates.DeployTemplate, 'list')
+    def test_deploy_templates_detailed(self, mock_list):
+        result = self.proxy.deploy_templates(details=True, query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, detail=True, query=1)
+
+    @mock.patch.object(deploy_templates.DeployTemplate, 'list')
+    def test_deploy_templates_not_detailed(self, mock_list):
+        result = self.proxy.deploy_templates(query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, query=1)
+
+    def test_create_deploy_template(self):
+        self.verify_create(
+            self.proxy.create_deploy_template,
+            deploy_templates.DeployTemplate,
+        )
+
+    def test_find_deploy_template(self):
+        self.verify_find(
+            self.proxy.find_deploy_template,
+            deploy_templates.DeployTemplate,
+            expected_kwargs={'details': True},
+        )
+
+    def test_get_deploy_template(self):
+        self.verify_get(
+            self.proxy.get_deploy_template,
+            deploy_templates.DeployTemplate,
+            mock_method=_MOCK_METHOD,
+            expected_kwargs={'fields': None},
+        )
+
+    def test_delete_deploy_template(self):
+        self.verify_delete(
+            self.proxy.delete_deploy_template,
+            deploy_templates.DeployTemplate,
+            False,
+        )
+
+    def test_delete_deploy_template_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_deploy_template,
+            deploy_templates.DeployTemplate,
+            True,
+        )
+
+
 class TestMisc(TestBaremetalProxy):
     @mock.patch.object(node.Node, 'fetch', autospec=True)
     def test__get_with_fields_none(self, mock_fetch):
@@ -349,7 +416,7 @@ class TestMisc(TestBaremetalProxy):
 @mock.patch.object(_proxy.Proxy, 'get_node', autospec=True)
 class TestWaitForNodesProvisionState(base.TestCase):
     def setUp(self):
-        super(TestWaitForNodesProvisionState, self).setUp()
+        super().setUp()
         self.session = mock.Mock()
         self.proxy = _proxy.Proxy(self.session)
 
@@ -444,3 +511,49 @@ class TestWaitForNodesProvisionState(base.TestCase):
         self.assertEqual(['1'], [x.id for x in result.success])
         self.assertEqual(['3'], [x.id for x in result.timeout])
         self.assertEqual(['2'], [x.id for x in result.failure])
+
+
+class TestInspectionRules(TestBaremetalProxy):
+    @mock.patch.object(inspection_rules.InspectionRule, 'list')
+    def test_inspection_rules_detailed(self, mock_list):
+        result = self.proxy.inspection_rules(details=True, query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, details=True, query=1)
+
+    @mock.patch.object(inspection_rules.InspectionRule, 'list')
+    def test_inspection_rules_not_detailed(self, mock_list):
+        result = self.proxy.inspection_rules(query=1)
+        self.assertIs(result, mock_list.return_value)
+        mock_list.assert_called_once_with(self.proxy, query=1)
+
+    def test_create_inspection_rule(self):
+        self.verify_create(
+            self.proxy.create_inspection_rule, inspection_rules.InspectionRule
+        )
+
+    def test_get_inspection_rule(self):
+        self.verify_get(
+            self.proxy.get_inspection_rule,
+            inspection_rules.InspectionRule,
+            mock_method=_MOCK_METHOD,
+            expected_kwargs={'fields': None},
+        )
+
+    def test_update_inspection_rule(self):
+        self.verify_update(
+            self.proxy.update_inspection_rule, inspection_rules.InspectionRule
+        )
+
+    def test_delete_inspection_rule(self):
+        self.verify_delete(
+            self.proxy.delete_inspection_rule,
+            inspection_rules.InspectionRule,
+            False,
+        )
+
+    def test_delete_inspection_rule_ignore(self):
+        self.verify_delete(
+            self.proxy.delete_inspection_rule,
+            inspection_rules.InspectionRule,
+            True,
+        )
