@@ -11,21 +11,18 @@
 # under the License.
 
 from openstack.block_storage.v3 import availability_zone as az
-
 from openstack.tests.unit import base
 
+
 IDENTIFIER = 'IDENTIFIER'
 EXAMPLE = {
     "id": IDENTIFIER,
-    "zoneState": {
-        "available": True
-    },
-    "zoneName": "zone1"
+    "zoneState": {"available": True},
+    "zoneName": "zone1",
 }
 
 
 class TestAvailabilityZone(base.TestCase):
-
     def test_basic(self):
         sot = az.AvailabilityZone()
         self.assertEqual('availabilityZoneInfo', sot.resources_key)
