@@ -11,6 +11,7 @@
 # under the License.
 
 from unittest import mock
+import warnings
 
 from openstack.block_storage.v3 import _proxy
 from openstack.block_storage.v3 import backup
@@ -25,11 +26,13 @@ from openstack.block_storage.v3 import resource_filter
 from openstack.block_storage.v3 import service
 from openstack.block_storage.v3 import snapshot
 from openstack.block_storage.v3 import stats
+from openstack.block_storage.v3 import transfer
 from openstack.block_storage.v3 import type
 from openstack.block_storage.v3 import volume
 from openstack.identity.v3 import project
 from openstack import proxy as proxy_base
 from openstack.tests.unit import test_proxy_base
+from openstack import warnings as os_warnings
 
 
 class TestVolumeProxy(test_proxy_base.TestProxyBase):
@@ -72,13 +75,26 @@ class TestVolume(TestVolumeProxy):
     def test_volume_create_attrs(self):
         self.verify_create(self.proxy.create_volume, volume.Volume)
 
-    def test_volume_delete(self):
-        self.verify_delete(self.proxy.delete_volume, volume.Volume, False)
+    @mock.patch(
+        'openstack.utils.supports_microversion',
+        autospec=True,
+        return_value=False,
+    )
+    def test_volume_delete(self, mock_mv):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.delete",
+            self.proxy.delete_volume,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_kwargs={"params": {"cascade": False}},
+        )
 
-    def test_volume_delete_ignore(self):
-        self.verify_delete(self.proxy.delete_volume, volume.Volume, True)
-
-    def test_volume_delete_force(self):
+    @mock.patch(
+        'openstack.utils.supports_microversion',
+        autospec=True,
+        return_value=False,
+    )
+    def test_volume_delete_force(self, mock_mv):
         self._verify(
             "openstack.block_storage.v3.volume.Volume.force_delete",
             self.proxy.delete_volume,
@@ -87,6 +103,24 @@ class TestVolume(TestVolumeProxy):
             expected_args=[self.proxy],
         )
 
+    @mock.patch(
+        'openstack.utils.supports_microversion',
+        autospec=True,
+        return_value=True,
+    )
+    def test_volume_delete_force_v323(self, mock_mv):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.delete",
+            self.proxy.delete_volume,
+            method_args=["value"],
+            method_kwargs={"force": True},
+            expected_args=[self.proxy],
+            expected_kwargs={"params": {"cascade": False, "force": True}},
+        )
+
+    def test_volume_update(self):
+        self.verify_update(self.proxy.update_volume, volume.Volume)
+
     def test_get_volume_metadata(self):
         self._verify(
             "openstack.block_storage.v3.volume.Volume.fetch_metadata",
@@ -124,8 +158,16 @@ class TestVolume(TestVolumeProxy):
         self.verify_wait_for_status(
             self.proxy.wait_for_status,
             method_args=[value],
-            expected_args=[self.proxy, value, 'available', ['error'], 2, 120],
-            expected_kwargs={'callback': None},
+            expected_args=[
+                self.proxy,
+                value,
+                'available',
+                ['error'],
+                2,
+                None,
+                'status',
+                None,
+            ],
         )
 
 
@@ -195,8 +237,41 @@ class TestGroup(TestVolumeProxy):
     def test_group_update(self):
         self.verify_update(self.proxy.update_group, group.Group)
 
-    def reset_group_state(self):
-        self._verify(self.proxy.reset_group_state, group.Group)
+    def test_group_reset_status(self):
+        self._verify(
+            "openstack.block_storage.v3.group.Group.reset_status",
+            self.proxy.reset_group_status,
+            method_args=["value", "new_status"],
+            expected_args=[self.proxy, "new_status"],
+        )
+
+    def test_group_enable_replication(self):
+        self._verify(
+            "openstack.block_storage.v3.group.Group.enable_replication",
+            self.proxy.enable_group_replication,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_group_disable_replication(self):
+        self._verify(
+            "openstack.block_storage.v3.group.Group.disable_replication",
+            self.proxy.disable_group_replication,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_group_failover_replication(self):
+        self._verify(
+            "openstack.block_storage.v3.group.Group.failover_replication",
+            self.proxy.failover_group_replication,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_kwargs={
+                'allowed_attached_volume': False,
+                'secondary_backend_id': None,
+            },
+        )
 
 
 class TestGroupSnapshot(TestVolumeProxy):
@@ -357,6 +432,33 @@ class TestService(TestVolumeProxy):
             expected_args=[self.proxy],
         )
 
+    def test_set_service_log_levels(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.set_log_levels',
+            self.proxy.set_service_log_levels,
+            method_kwargs={"level": service.Level.INFO},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                "level": service.Level.INFO,
+                "binary": None,
+                "server": None,
+                "prefix": None,
+            },
+        )
+
+    def test_get_service_log_level(self):
+        self._verify(
+            'openstack.block_storage.v3.service.Service.get_log_levels',
+            self.proxy.get_service_log_levels,
+            method_args=[],
+            expected_args=[self.proxy],
+            expected_kwargs={
+                "binary": None,
+                "server": None,
+                "prefix": None,
+            },
+        )
+
     def test_failover_service(self):
         self._verify(
             'openstack.block_storage.v3.service.Service.failover',
@@ -381,6 +483,22 @@ class TestVolumeActions(TestVolumeProxy):
             expected_args=[self.proxy, "new-size"],
         )
 
+    def test_complete_extend(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.complete_extend",
+            self.proxy.complete_volume_extend,
+            method_args=["value"],
+            expected_args=[self.proxy, False],
+        )
+
+    def test_complete_extend_error(self):
+        self._verify(
+            "openstack.block_storage.v3.volume.Volume.complete_extend",
+            self.proxy.complete_volume_extend,
+            method_args=["value", True],
+            expected_args=[self.proxy, True],
+        )
+
     def test_volume_set_readonly_no_argument(self):
         self._verify(
             "openstack.block_storage.v3.volume.Volume.set_readonly",
@@ -675,6 +793,9 @@ class TestBackup(TestVolumeProxy):
             expected_args=[self.proxy],
         )
 
+    def test_backup_update(self):
+        self.verify_update(self.proxy.update_backup, backup.Backup)
+
     def test_backup_create_attrs(self):
         # NOTE: mock has_service
         self.proxy._connection = mock.Mock()
@@ -694,14 +815,46 @@ class TestBackup(TestVolumeProxy):
             expected_kwargs={'volume_id': 'vol_id', 'name': 'name'},
         )
 
-    def test_backup_reset(self):
+    def test_backup_reset_status(self):
         self._verify(
-            "openstack.block_storage.v3.backup.Backup.reset",
-            self.proxy.reset_backup,
+            "openstack.block_storage.v3.backup.Backup.reset_status",
+            self.proxy.reset_backup_status,
             method_args=["value", "new_status"],
             expected_args=[self.proxy, "new_status"],
         )
 
+    def test_backup_get_metadata(self):
+        self._verify(
+            "openstack.block_storage.v3.backup.Backup.fetch_metadata",
+            self.proxy.get_backup_metadata,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_result=volume.Volume(id="value", metadata={}),
+        )
+
+    def test_backup_set_metadata(self):
+        kwargs = {"a": "1", "b": "2"}
+        id = "an_id"
+        self._verify(
+            "openstack.block_storage.v3.backup.Backup.set_metadata",
+            self.proxy.set_backup_metadata,
+            method_args=[id],
+            method_kwargs=kwargs,
+            method_result=volume.Volume.existing(id=id, metadata=kwargs),
+            expected_args=[self.proxy],
+            expected_kwargs={'metadata': kwargs},
+            expected_result=volume.Volume.existing(id=id, metadata=kwargs),
+        )
+
+    def test_backup_delete_metadata(self):
+        self._verify(
+            "openstack.block_storage.v3.backup.Backup.delete_metadata_item",
+            self.proxy.delete_backup_metadata,
+            expected_result=None,
+            method_args=["value", ["key"]],
+            expected_args=[self.proxy, "key"],
+        )
+
 
 class TestSnapshot(TestVolumeProxy):
     def test_snapshot_get(self):
@@ -757,15 +910,15 @@ class TestSnapshot(TestVolumeProxy):
             expected_args=[self.proxy],
         )
 
-    def test_reset(self):
+    def test_snapshot_reset_status(self):
         self._verify(
-            "openstack.block_storage.v3.snapshot.Snapshot.reset",
-            self.proxy.reset_snapshot,
+            "openstack.block_storage.v3.snapshot.Snapshot.reset_status",
+            self.proxy.reset_snapshot_status,
             method_args=["value", "new_status"],
             expected_args=[self.proxy, "new_status"],
         )
 
-    def test_set_status(self):
+    def test_snapshot_set_status(self):
         self._verify(
             "openstack.block_storage.v3.snapshot.Snapshot.set_status",
             self.proxy.set_snapshot_status,
@@ -773,7 +926,7 @@ class TestSnapshot(TestVolumeProxy):
             expected_args=[self.proxy, "new_status", None],
         )
 
-    def test_set_status_percentage(self):
+    def test_snapshot_set_status_percentage(self):
         self._verify(
             "openstack.block_storage.v3.snapshot.Snapshot.set_status",
             self.proxy.set_snapshot_status,
@@ -781,6 +934,33 @@ class TestSnapshot(TestVolumeProxy):
             expected_args=[self.proxy, "new_status", "per"],
         )
 
+    def test_snapshot_manage(self):
+        kwargs = {
+            "volume_id": "fake_id",
+            "remote_source": "fake_volume",
+            "snapshot_name": "fake_snap",
+            "description": "test_snap",
+            "property": {"k": "v"},
+        }
+        self._verify(
+            "openstack.block_storage.v3.snapshot.Snapshot.manage",
+            self.proxy.manage_snapshot,
+            method_kwargs=kwargs,
+            method_result=snapshot.Snapshot(id="fake_id"),
+            expected_args=[self.proxy],
+            expected_kwargs=kwargs,
+            expected_result=snapshot.Snapshot(id="fake_id"),
+        )
+
+    def test_snapshot_unmanage(self):
+        self._verify(
+            "openstack.block_storage.v3.snapshot.Snapshot.unmanage",
+            self.proxy.unmanage_snapshot,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_result=None,
+        )
+
     def test_get_snapshot_metadata(self):
         self._verify(
             "openstack.block_storage.v3.snapshot.Snapshot.fetch_metadata",
@@ -814,24 +994,6 @@ class TestSnapshot(TestVolumeProxy):
             expected_args=[self.proxy, "key"],
         )
 
-    def test_manage_snapshot(self):
-        kwargs = {
-            "volume_id": "fake_id",
-            "remote_source": "fake_volume",
-            "snapshot_name": "fake_snap",
-            "description": "test_snap",
-            "property": {"k": "v"},
-        }
-        self._verify(
-            "openstack.block_storage.v3.snapshot.Snapshot.manage",
-            self.proxy.manage_snapshot,
-            method_kwargs=kwargs,
-            method_result=snapshot.Snapshot(id="fake_id"),
-            expected_args=[self.proxy],
-            expected_kwargs=kwargs,
-            expected_result=snapshot.Snapshot(id="fake_id"),
-        )
-
 
 class TestType(TestVolumeProxy):
     def test_type_get(self):
@@ -951,6 +1113,39 @@ class TestType(TestVolumeProxy):
         )
 
 
+class TestTransfer(TestVolumeProxy):
+    def test_transfer_create(self):
+        self.verify_create(self.proxy.create_transfer, transfer.Transfer)
+
+    def test_transfer_delete(self):
+        self.verify_delete(
+            self.proxy.delete_transfer, transfer.Transfer, False
+        )
+
+    def test_transfer_get(self):
+        self.verify_get(self.proxy.get_transfer, transfer.Transfer)
+
+    def test_transfer_find(self):
+        self.verify_find(self.proxy.find_transfer, transfer.Transfer)
+
+    @mock.patch(
+        'openstack.utils.supports_microversion',
+        autospec=True,
+        return_value=False,
+    )
+    def test_transfers(self, mock_mv):
+        self.verify_list(self.proxy.transfers, transfer.Transfer)
+
+    def test_accept_transfer(self):
+        self._verify(
+            'openstack.block_storage.v3.transfer.Transfer.accept',
+            self.proxy.accept_transfer,
+            method_args=['value', 'auth_key'],
+            expected_args=[self.proxy],
+            expected_kwargs={'auth_key': 'auth_key'},
+        )
+
+
 class TestQuotaClassSet(TestVolumeProxy):
     def test_quota_class_set_get(self):
         self.verify_get(
@@ -1009,14 +1204,15 @@ class TestQuotaSet(TestVolumeProxy):
         )
 
     def test_quota_set_get_defaults(self):
+        project_id = 'prj'
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set_defaults,
-            method_args=['prj'],
+            method_args=[project_id],
             expected_args=[
                 self.proxy,
                 False,
-                '/os-quota-sets/defaults',
+                f'/os-quota-sets/{project_id}/defaults',
                 None,
                 False,
             ],
@@ -1050,3 +1246,29 @@ class TestQuotaSet(TestVolumeProxy):
             expected_kwargs={'project_id': 'prj', 'volumes': 123},
         )
         mock_get.assert_called_once_with(project.Project, 'prj')
+
+    @mock.patch.object(proxy_base.Proxy, "_get_resource")
+    def test_quota_set_update__legacy(self, mock_get):
+        fake_quota_set = quota_set.QuotaSet(project_id='prj')
+        mock_get.side_effect = [fake_quota_set]
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+
+            self._verify(
+                'openstack.resource.Resource.commit',
+                self.proxy.update_quota_set,
+                method_args=[fake_quota_set],
+                method_kwargs={'ram': 123},
+                expected_args=[self.proxy],
+                expected_kwargs={},
+            )
+
+            self.assertEqual(1, len(w))
+            self.assertEqual(
+                os_warnings.RemovedInSDK50Warning,
+                w[-1].category,
+            )
+            self.assertIn(
+                "The signature of 'update_quota_set' has changed ",
+                str(w[-1]),
+            )
