@@ -30,7 +30,7 @@ TYPE = {
 
 class TestType(base.TestCase):
     def setUp(self):
-        super(TestType, self).setUp()
+        super().setUp()
         self.extra_specs_result = {"extra_specs": {"go": "cubs", "boo": "sox"}}
         self.resp = mock.Mock()
         self.resp.body = None
@@ -98,7 +98,7 @@ class TestType(base.TestCase):
             exceptions.BadRequestException,
             sot.set_extra_specs,
             sess,
-            **set_specs
+            **set_specs,
         )
 
     def test_delete_extra_specs(self):
@@ -150,7 +150,7 @@ class TestType(base.TestCase):
         )
 
         self.sess.get.assert_called_with(
-            "types/%s/os-volume-type-access" % sot.id
+            f"types/{sot.id}/os-volume-type-access"
         )
 
     def test_add_private_access(self):
@@ -158,7 +158,7 @@ class TestType(base.TestCase):
 
         self.assertIsNone(sot.add_private_access(self.sess, "a"))
 
-        url = "types/%s/action" % sot.id
+        url = f"types/{sot.id}/action"
         body = {"addProjectAccess": {"project": "a"}}
         self.sess.post.assert_called_with(url, json=body)
 
@@ -167,6 +167,6 @@ class TestType(base.TestCase):
 
         self.assertIsNone(sot.remove_private_access(self.sess, "a"))
 
-        url = "types/%s/action" % sot.id
+        url = f"types/{sot.id}/action"
         body = {"removeProjectAccess": {"project": "a"}}
         self.sess.post.assert_called_with(url, json=body)
