@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -86,13 +84,13 @@ class TestUtils(base.TestCase):
     def test__filter_list_unicode(self):
         el1 = dict(
             id=100,
-            name=u'中文',
+            name='中文',
             last='duck',
             other=dict(category='duck', financial=dict(status='poor')),
         )
         el2 = dict(
             id=200,
-            name=u'中文',
+            name='中文',
             last='trump',
             other=dict(category='human', financial=dict(status='rich')),
         )
@@ -104,7 +102,7 @@ class TestUtils(base.TestCase):
         )
         data = [el1, el2, el3]
         ret = _utils._filter_list(
-            data, u'中文', {'other': {'financial': {'status': 'rich'}}}
+            data, '中文', {'other': {'financial': {'status': 'rich'}}}
         )
         self.assertEqual([el2], ret)
 
@@ -333,7 +331,7 @@ class TestUtils(base.TestCase):
         # if the use_direct_get flag is set to False(default).
         uuid = uuid4().hex
         resource = 'network'
-        func = 'search_%ss' % resource
+        func = f'search_{resource}s'
         filters = {}
         with mock.patch.object(self.cloud, func) as search:
             _utils._get_entity(self.cloud, resource, uuid, filters)
@@ -345,7 +343,7 @@ class TestUtils(base.TestCase):
         self.cloud.use_direct_get = True
         name = 'name_no_uuid'
         resource = 'network'
-        func = 'search_%ss' % resource
+        func = f'search_{resource}s'
         filters = {}
         with mock.patch.object(self.cloud, func) as search:
             _utils._get_entity(self.cloud, resource, name, filters)
@@ -365,7 +363,7 @@ class TestUtils(base.TestCase):
             'security_group',
         ]
         for r in resources:
-            f = 'get_%s_by_id' % r
+            f = f'get_{r}_by_id'
             with mock.patch.object(self.cloud, f) as get:
                 _utils._get_entity(self.cloud, r, uuid, {})
                 get.assert_called_once_with(uuid)
@@ -385,7 +383,7 @@ class TestUtils(base.TestCase):
         filters = {}
         name = 'name_no_uuid'
         for r in resources:
-            f = 'search_%ss' % r
+            f = f'search_{r}s'
             with mock.patch.object(self.cloud, f) as search:
                 _utils._get_entity(self.cloud, r, name, {})
                 search.assert_called_once_with(name, filters)
@@ -402,5 +400,5 @@ class TestUtils(base.TestCase):
             'security_group',
         ]
         for r in resources:
-            self.assertTrue(hasattr(self.cloud, 'get_%s_by_id' % r))
-            self.assertTrue(hasattr(self.cloud, 'search_%ss' % r))
+            self.assertTrue(hasattr(self.cloud, f'get_{r}_by_id'))
+            self.assertTrue(hasattr(self.cloud, f'search_{r}s'))
