@@ -17,7 +17,7 @@ from unittest import mock
 
 import testtools
 
-import openstack.cloud.openstackcloud as oc_oc
+from openstack.cloud import _object_store
 from openstack import exceptions
 from openstack.object_store.v1 import _proxy
 from openstack.object_store.v1 import container
@@ -28,17 +28,13 @@ from openstack import utils
 
 class BaseTestObject(base.TestCase):
     def setUp(self):
-        super(BaseTestObject, self).setUp()
+        super().setUp()
 
         self.container = self.getUniqueString()
         self.object = self.getUniqueString()
         self.endpoint = self.cloud.object_store.get_endpoint()
-        self.container_endpoint = '{endpoint}/{container}'.format(
-            endpoint=self.endpoint, container=self.container
-        )
-        self.object_endpoint = '{endpoint}/{object}'.format(
-            endpoint=self.container_endpoint, object=self.object
-        )
+        self.container_endpoint = f'{self.endpoint}/{self.container}'
+        self.object_endpoint = f'{self.container_endpoint}/{self.object}'
 
     def _compare_containers(self, exp, real):
         self.assertDictEqual(
@@ -107,7 +103,7 @@ class TestObject(BaseTestObject):
                         'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                         'Content-Length': '0',
                         'Content-Type': 'text/html; charset=UTF-8',
-                        'x-container-read': oc_oc.OBJECT_CONTAINER_ACLS[
+                        'x-container-read': _object_store.OBJECT_CONTAINER_ACLS[
                             'public'
                         ],
                     },
@@ -203,7 +199,9 @@ class TestObject(BaseTestObject):
         self.assert_calls()
 
     def test_update_container(self):
-        headers = {'x-container-read': oc_oc.OBJECT_CONTAINER_ACLS['public']}
+        headers = {
+            'x-container-read': _object_store.OBJECT_CONTAINER_ACLS['public']
+        }
         self.register_uris(
             [
                 dict(
@@ -245,7 +243,7 @@ class TestObject(BaseTestObject):
                     status_code=204,
                     validate=dict(
                         headers={
-                            'x-container-read': oc_oc.OBJECT_CONTAINER_ACLS[
+                            'x-container-read': _object_store.OBJECT_CONTAINER_ACLS[
                                 'public'
                             ]
                         }
@@ -267,7 +265,7 @@ class TestObject(BaseTestObject):
                     status_code=204,
                     validate=dict(
                         headers={
-                            'x-container-read': oc_oc.OBJECT_CONTAINER_ACLS[
+                            'x-container-read': _object_store.OBJECT_CONTAINER_ACLS[
                                 'private'
                             ]
                         }
@@ -296,7 +294,7 @@ class TestObject(BaseTestObject):
                     uri=self.container_endpoint,
                     headers={
                         'x-container-read': str(
-                            oc_oc.OBJECT_CONTAINER_ACLS['public']
+                            _object_store.OBJECT_CONTAINER_ACLS['public']
                         )
                     },
                 )
@@ -328,13 +326,13 @@ class TestObject(BaseTestObject):
         )
         with testtools.ExpectedException(
             exceptions.SDKException,
-            "Container not found: %s" % self.container,
+            f"Container not found: {self.container}",
         ):
             self.cloud.get_container_access(self.container)
 
     def test_list_containers(self):
-        endpoint = '{endpoint}/'.format(endpoint=self.endpoint)
-        containers = [{u'count': 0, u'bytes': 0, u'name': self.container}]
+        endpoint = f'{self.endpoint}/'
+        containers = [{'count': 0, 'bytes': 0, 'name': self.container}]
 
         self.register_uris(
             [
@@ -354,7 +352,7 @@ class TestObject(BaseTestObject):
             self._compare_containers(a, b)
 
     def test_list_containers_exception(self):
-        endpoint = '{endpoint}/'.format(endpoint=self.endpoint)
+        endpoint = f'{self.endpoint}/'
         self.register_uris(
             [
                 dict(
@@ -592,17 +590,15 @@ class TestObject(BaseTestObject):
         self.assert_calls()
 
     def test_list_objects(self):
-        endpoint = '{endpoint}?format=json'.format(
-            endpoint=self.container_endpoint
-        )
+        endpoint = f'{self.container_endpoint}?format=json'
 
         objects = [
             {
-                u'bytes': 20304400896,
-                u'last_modified': u'2016-12-15T13:34:13.650090',
-                u'hash': u'daaf9ed2106d09bba96cf193d866445e',
-                u'name': self.object,
-                u'content_type': u'application/octet-stream',
+                'bytes': 20304400896,
+                'last_modified': '2016-12-15T13:34:13.650090',
+                'hash': 'daaf9ed2106d09bba96cf193d866445e',
+                'name': self.object,
+                'content_type': 'application/octet-stream',
             }
         ]
 
@@ -617,17 +613,15 @@ class TestObject(BaseTestObject):
             self._compare_objects(a, b)
 
     def test_list_objects_with_prefix(self):
-        endpoint = '{endpoint}?format=json&prefix=test'.format(
-            endpoint=self.container_endpoint
-        )
+        endpoint = f'{self.container_endpoint}?format=json&prefix=test'
 
         objects = [
             {
-                u'bytes': 20304400896,
-                u'last_modified': u'2016-12-15T13:34:13.650090',
-                u'hash': u'daaf9ed2106d09bba96cf193d866445e',
-                u'name': self.object,
-                u'content_type': u'application/octet-stream',
+                'bytes': 20304400896,
+                'last_modified': '2016-12-15T13:34:13.650090',
+                'hash': 'daaf9ed2106d09bba96cf193d866445e',
+                'name': self.object,
+                'content_type': 'application/octet-stream',
             }
         ]
 
@@ -642,9 +636,7 @@ class TestObject(BaseTestObject):
             self._compare_objects(a, b)
 
     def test_list_objects_exception(self):
-        endpoint = '{endpoint}?format=json'.format(
-            endpoint=self.container_endpoint
-        )
+        endpoint = f'{self.container_endpoint}?format=json'
         self.register_uris(
             [
                 dict(
@@ -879,7 +871,7 @@ class TestObject(BaseTestObject):
 
 class TestObjectUploads(BaseTestObject):
     def setUp(self):
-        super(TestObjectUploads, self).setUp()
+        super().setUp()
 
         self.content = self.getUniqueString().encode('latin-1')
         self.object_file = tempfile.NamedTemporaryFile(delete=False)
@@ -901,20 +893,12 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='HEAD',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=404,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                     validate=dict(
                         headers={
@@ -970,11 +954,7 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                     validate=dict(
                         headers={
@@ -1006,11 +986,7 @@ class TestObjectUploads(BaseTestObject):
             ),
             dict(
                 method='HEAD',
-                uri='{endpoint}/{container}/{object}'.format(
-                    endpoint=self.endpoint,
-                    container=self.container,
-                    object=self.object,
-                ),
+                uri=f'{self.endpoint}/{self.container}/{self.object}',
                 status_code=404,
             ),
         ]
@@ -1019,12 +995,7 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/{index:0>6}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                        index=index,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/{index:0>6}',
                     status_code=201,
                 )
                 for index, offset in enumerate(
@@ -1036,17 +1007,11 @@ class TestObjectUploads(BaseTestObject):
         uris_to_mock.append(
             dict(
                 method='PUT',
-                uri='{endpoint}/{container}/{object}'.format(
-                    endpoint=self.endpoint,
-                    container=self.container,
-                    object=self.object,
-                ),
+                uri=f'{self.endpoint}/{self.container}/{self.object}',
                 status_code=201,
                 validate=dict(
                     headers={
-                        'x-object-manifest': '{container}/{object}'.format(
-                            container=self.container, object=self.object
-                        ),
+                        'x-object-manifest': f'{self.container}/{self.object}',
                         'x-object-meta-x-sdk-md5': self.md5,
                         'x-object-meta-x-sdk-sha256': self.sha256,
                     }
@@ -1086,11 +1051,7 @@ class TestObjectUploads(BaseTestObject):
             ),
             dict(
                 method='HEAD',
-                uri='{endpoint}/{container}/{object}'.format(
-                    endpoint=self.endpoint,
-                    container=self.container,
-                    object=self.object,
-                ),
+                uri=f'{self.endpoint}/{self.container}/{self.object}',
                 status_code=404,
             ),
         ]
@@ -1099,14 +1060,9 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/{index:0>6}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                        index=index,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/{index:0>6}',
                     status_code=201,
-                    headers=dict(Etag='etag{index}'.format(index=index)),
+                    headers=dict(Etag=f'etag{index}'),
                 )
                 for index, offset in enumerate(
                     range(0, len(self.content), max_file_size)
@@ -1117,11 +1073,7 @@ class TestObjectUploads(BaseTestObject):
         uris_to_mock.append(
             dict(
                 method='PUT',
-                uri='{endpoint}/{container}/{object}'.format(
-                    endpoint=self.endpoint,
-                    container=self.container,
-                    object=self.object,
-                ),
+                uri=f'{self.endpoint}/{self.container}/{self.object}',
                 status_code=201,
                 validate=dict(
                     params={'multipart-manifest', 'put'},
@@ -1151,37 +1103,27 @@ class TestObjectUploads(BaseTestObject):
                 'header mismatch in manifest call',
             )
 
-        base_object = '/{container}/{object}'.format(
-            container=self.container, object=self.object
-        )
+        base_object = f'/{self.container}/{self.object}'
 
         self.assertEqual(
             [
                 {
-                    'path': "{base_object}/000000".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000000",
                     'size_bytes': 25,
                     'etag': 'etag0',
                 },
                 {
-                    'path': "{base_object}/000001".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000001",
                     'size_bytes': 25,
                     'etag': 'etag1',
                 },
                 {
-                    'path': "{base_object}/000002".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000002",
                     'size_bytes': 25,
                     'etag': 'etag2',
                 },
                 {
-                    'path': "{base_object}/000003".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000003",
                     'size_bytes': len(self.object) - 75,
                     'etag': 'etag3',
                 },
@@ -1208,11 +1150,7 @@ class TestObjectUploads(BaseTestObject):
             ),
             dict(
                 method='HEAD',
-                uri='{endpoint}/{container}/{object}'.format(
-                    endpoint=self.endpoint,
-                    container=self.container,
-                    object=self.object,
-                ),
+                uri=f'{self.endpoint}/{self.container}/{self.object}',
                 status_code=404,
             ),
         ]
@@ -1221,14 +1159,9 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/{index:0>6}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                        index=index,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/{index:0>6}',
                     status_code=201,
-                    headers=dict(Etag='etag{index}'.format(index=index)),
+                    headers=dict(Etag=f'etag{index}'),
                 )
                 for index, offset in enumerate(
                     range(0, len(self.content), max_file_size)
@@ -1241,11 +1174,7 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=400,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1257,11 +1186,7 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=400,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1273,11 +1198,7 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1309,37 +1230,27 @@ class TestObjectUploads(BaseTestObject):
                 'header mismatch in manifest call',
             )
 
-        base_object = '/{container}/{object}'.format(
-            container=self.container, object=self.object
-        )
+        base_object = f'/{self.container}/{self.object}'
 
         self.assertEqual(
             [
                 {
-                    'path': "{base_object}/000000".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000000",
                     'size_bytes': 25,
                     'etag': 'etag0',
                 },
                 {
-                    'path': "{base_object}/000001".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000001",
                     'size_bytes': 25,
                     'etag': 'etag1',
                 },
                 {
-                    'path': "{base_object}/000002".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000002",
                     'size_bytes': 25,
                     'etag': 'etag2',
                 },
                 {
-                    'path': "{base_object}/000003".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000003",
                     'size_bytes': len(self.object) - 75,
                     'etag': 'etag3',
                 },
@@ -1367,11 +1278,7 @@ class TestObjectUploads(BaseTestObject):
             ),
             dict(
                 method='HEAD',
-                uri='{endpoint}/{container}/{object}'.format(
-                    endpoint=self.endpoint,
-                    container=self.container,
-                    object=self.object,
-                ),
+                uri=f'{self.endpoint}/{self.container}/{self.object}',
                 status_code=404,
             ),
         ]
@@ -1380,14 +1287,9 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/{index:0>6}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                        index=index,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/{index:0>6}',
                     status_code=201,
-                    headers=dict(Etag='etag{index}'.format(index=index)),
+                    headers=dict(Etag=f'etag{index}'),
                 )
                 for index, offset in enumerate(
                     range(0, len(self.content), max_file_size)
@@ -1400,11 +1302,7 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=400,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1416,11 +1314,7 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=400,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1432,11 +1326,7 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=400,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1457,9 +1347,7 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='GET',
-                    uri='{endpoint}/images?format=json&prefix={prefix}'.format(
-                        endpoint=self.endpoint, prefix=self.object
-                    ),
+                    uri=f'{self.endpoint}/images?format=json&prefix={self.object}',
                     complete_qs=True,
                     json=[
                         {
@@ -1473,9 +1361,7 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='HEAD',
-                    uri='{endpoint}/images/{object}'.format(
-                        endpoint=self.endpoint, object=self.object
-                    ),
+                    uri=f'{self.endpoint}/images/{self.object}',
                     headers={
                         'X-Timestamp': '1429036140.50253',
                         'X-Trans-Id': 'txbbb825960a3243b49a36f-005a0dadaedfw1',
@@ -1493,9 +1379,7 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='DELETE',
-                    uri='{endpoint}/images/{object}'.format(
-                        endpoint=self.endpoint, object=self.object
-                    ),
+                    uri=f'{self.endpoint}/images/{self.object}',
                 ),
             ]
         )
@@ -1534,56 +1418,32 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='HEAD',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=404,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000000'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000000',
                     status_code=201,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000001'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000001',
                     status_code=201,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000002'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000002',
                     status_code=201,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000003'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000003',
                     status_code=501,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                 ),
             ]
@@ -1617,69 +1477,41 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='HEAD',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=404,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000000'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000000',
                     headers={'etag': 'etag0'},
                     status_code=201,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000001'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000001',
                     headers={'etag': 'etag1'},
                     status_code=201,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000002'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000002',
                     headers={'etag': 'etag2'},
                     status_code=201,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000003'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000003',
                     status_code=501,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}/000003'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}/000003',
                     status_code=201,
                     headers={'etag': 'etag3'},
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                     validate=dict(
                         params={'multipart-manifest', 'put'},
@@ -1709,37 +1541,27 @@ class TestObjectUploads(BaseTestObject):
                 'header mismatch in manifest call',
             )
 
-        base_object = '/{container}/{object}'.format(
-            container=self.container, object=self.object
-        )
+        base_object = f'/{self.container}/{self.object}'
 
         self.assertEqual(
             [
                 {
-                    'path': "{base_object}/000000".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000000",
                     'size_bytes': 25,
                     'etag': 'etag0',
                 },
                 {
-                    'path': "{base_object}/000001".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000001",
                     'size_bytes': 25,
                     'etag': 'etag1',
                 },
                 {
-                    'path': "{base_object}/000002".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000002",
                     'size_bytes': 25,
                     'etag': 'etag2',
                 },
                 {
-                    'path': "{base_object}/000003".format(
-                        base_object=base_object
-                    ),
+                    'path': f"{base_object}/000003",
                     'size_bytes': len(self.object) - 75,
                     'etag': 'etag3',
                 },
@@ -1760,20 +1582,12 @@ class TestObjectUploads(BaseTestObject):
                 ),
                 dict(
                     method='HEAD',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=200,
                 ),
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                     validate=dict(headers={}),
                 ),
@@ -1794,11 +1608,7 @@ class TestObjectUploads(BaseTestObject):
             [
                 dict(
                     method='PUT',
-                    uri='{endpoint}/{container}/{object}'.format(
-                        endpoint=self.endpoint,
-                        container=self.container,
-                        object=self.object,
-                    ),
+                    uri=f'{self.endpoint}/{self.container}/{self.object}',
                     status_code=201,
                     validate=dict(
                         headers={},
