@@ -34,9 +34,7 @@ fake_quota_set = {
 
 class TestQuotas(base.TestCase):
     def setUp(self, cloud_config_fixture='clouds.yaml'):
-        super(TestQuotas, self).setUp(
-            cloud_config_fixture=cloud_config_fixture
-        )
+        super().setUp(cloud_config_fixture=cloud_config_fixture)
 
     def test_update_quotas(self):
         project = self._get_project_data()
@@ -127,7 +125,6 @@ class TestQuotas(base.TestCase):
                         'compute',
                         'public',
                         append=['os-quota-sets', project.project_id],
-                        qs_elements=['usage=False'],
                     ),
                     json={'quota_set': fake_quota_set},
                 ),
@@ -258,7 +255,7 @@ class TestQuotas(base.TestCase):
 
     def test_neutron_update_quotas(self):
         project = self.mock_for_keystone_projects(
-            project_count=1, list_get=True
+            project_count=1, id_get=True
         )[0]
         self.register_uris(
             [
@@ -362,7 +359,7 @@ class TestQuotas(base.TestCase):
 
     def test_neutron_delete_quotas(self):
         project = self.mock_for_keystone_projects(
-            project_count=1, list_get=True
+            project_count=1, id_get=True
         )[0]
         self.register_uris(
             [
