@@ -19,6 +19,8 @@ from openstack.tests.unit import base
 
 
 class TestRoleAssignment(base.TestCase):
+    IS_INHERITED = False
+
     def _build_role_assignment_response(
         self, role_id, scope_type, scope_id, entity_type, entity_id
     ):
@@ -48,7 +50,7 @@ class TestRoleAssignment(base.TestCase):
         ]
 
     def setUp(self, cloud_config_fixture='clouds.yaml'):
-        super(TestRoleAssignment, self).setUp(cloud_config_fixture)
+        super().setUp(cloud_config_fixture)
         self.role_data = self._get_role_data()
         self.domain_data = self._get_domain_data()
         self.user_data = self._get_user_data(
@@ -119,8 +121,14 @@ class TestRoleAssignment(base.TestCase):
         append=None,
         base_url_append='v3',
         qs_elements=None,
+        inherited=False,
     ):
-        return super(TestRoleAssignment, self).get_mock_url(
+        if inherited:
+            base_url_append = base_url_append + '/OS-INHERIT'
+        if append and inherited:
+            append.append('inherited_to_projects')
+
+        return super().get_mock_url(
             service_type,
             interface,
             resource,
@@ -190,37 +198,56 @@ class TestRoleAssignment(base.TestCase):
                 ),
             ]
 
-    def __user_mocks(self, user_data, use_name, is_found=True):
+    def __user_mocks(
+        self, user_data, use_name, is_found=True, domain_data=None
+    ):
+        qs_elements = []
+        if domain_data:
+            qs_elements = ['domain_id=' + domain_data.domain_id]
         uri_mocks = []
         if not use_name:
             uri_mocks.append(
                 dict(
                     method='GET',
-                    uri=self.get_mock_url(resource='users'),
-                    status_code=200,
-                    json={
-                        'users': [user_data.json_response['user']]
-                        if is_found
-                        else []
-                    },
-                )
+                    uri=self.get_mock_url(
+                        resource='users',
+                        append=[user_data.user_id],
+                        # TODO(stephenfin): We shouldn't be passing domain ID
+                        # here since it's unnecessary, but that requires a much
+                        # larger rework of the Resource.find method.
+                        qs_elements=qs_elements,
+                    ),
+                    json=user_data.json_response if is_found else None,
+                    status_code=200 if is_found else 404,
+                ),
             )
         else:
-            uri_mocks.append(
+            uri_mocks += [
                 dict(
                     method='GET',
                     uri=self.get_mock_url(
                         resource='users',
-                        qs_elements=['name=' + user_data.name],
+                        append=[user_data.name],
+                        qs_elements=qs_elements,
+                    ),
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri=self.get_mock_url(
+                        resource='users',
+                        qs_elements=qs_elements + ['name=' + user_data.name],
                     ),
                     status_code=200,
                     json={
-                        'users': [user_data.json_response['user']]
-                        if is_found
-                        else []
+                        'users': (
+                            [user_data.json_response['user']]
+                            if is_found
+                            else []
+                        )
                     },
-                )
-            )
+                ),
+            ]
         return uri_mocks
 
     def _get_mock_role_query_urls(
@@ -267,7 +294,12 @@ class TestRoleAssignment(base.TestCase):
 
         if user_data:
             uri_mocks.extend(
-                self.__user_mocks(user_data, use_user_name, is_found=True)
+                self.__user_mocks(
+                    user_data,
+                    use_user_name,
+                    is_found=True,
+                    domain_data=domain_data,
+                )
             )
 
         if group_data:
@@ -314,6 +346,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -329,6 +362,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -341,6 +375,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -366,6 +401,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -381,6 +417,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -393,6 +430,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -415,6 +453,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -428,6 +467,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -453,6 +493,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -466,6 +507,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -490,6 +532,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -505,6 +548,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -517,6 +561,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -542,6 +587,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -557,6 +603,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -569,6 +616,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -592,6 +640,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -605,6 +654,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -630,6 +680,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -643,6 +694,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -668,6 +720,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -683,6 +736,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -695,6 +749,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -720,6 +775,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -735,6 +791,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -747,6 +804,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -769,6 +827,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -782,6 +841,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -807,6 +867,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -820,6 +881,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -844,6 +906,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -859,6 +922,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -871,6 +935,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -896,6 +961,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -911,6 +977,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -923,6 +990,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -946,6 +1014,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -959,6 +1028,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -984,6 +1054,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -997,6 +1068,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1022,6 +1094,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1037,6 +1110,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1049,6 +1123,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1074,6 +1149,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1089,6 +1165,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1101,6 +1178,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -1123,6 +1201,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1136,6 +1215,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1161,6 +1241,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1174,6 +1255,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1198,6 +1280,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1213,6 +1296,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1225,6 +1309,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1250,6 +1335,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1265,6 +1351,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1277,6 +1364,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1300,6 +1388,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1313,6 +1402,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1338,6 +1428,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1351,6 +1442,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 project=self.project_data.project_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1376,6 +1468,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1391,6 +1484,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1403,6 +1497,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1428,6 +1523,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1443,6 +1539,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1455,6 +1552,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -1477,6 +1575,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1490,6 +1589,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 user=self.user_data.user_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1515,6 +1615,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1528,6 +1629,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1552,6 +1654,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1567,6 +1670,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1579,6 +1683,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1604,6 +1709,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1619,6 +1725,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1631,6 +1738,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1654,6 +1762,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1667,6 +1776,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_id,
                 group=self.group_data.group_id,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1692,6 +1802,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1705,6 +1816,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_id,
+                inherited=self.IS_INHERITED,
             )
         )
         self.assert_calls()
@@ -1738,14 +1850,12 @@ class TestRoleAssignment(base.TestCase):
         )
         self.register_uris(uris)
 
-        with testtools.ExpectedException(
-            exceptions.SDKException,
-            'Role {0} not found'.format(self.role_data.role_name),
-        ):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.grant_role(
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1778,14 +1888,12 @@ class TestRoleAssignment(base.TestCase):
         )
         self.register_uris(uris)
 
-        with testtools.ExpectedException(
-            exceptions.SDKException,
-            'Role {0} not found'.format(self.role_data.role_name),
-        ):
+        with testtools.ExpectedException(exceptions.NotFoundException):
             self.cloud.revoke_role(
                 self.role_data.role_name,
                 group=self.group_data.group_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1798,7 +1906,10 @@ class TestRoleAssignment(base.TestCase):
             exceptions.SDKException,
             'Must specify either a user or a group',
         ):
-            self.cloud.grant_role(self.role_data.role_name)
+            self.cloud.grant_role(
+                self.role_data.role_name,
+                inherited=self.IS_INHERITED,
+            )
         self.assert_calls()
 
     def test_revoke_no_user_or_group_specified(self):
@@ -1806,46 +1917,13 @@ class TestRoleAssignment(base.TestCase):
             'role', self.role_data, 'role_name', [], use_name=True
         )
         self.register_uris(uris)
-        with testtools.ExpectedException(
-            exceptions.SDKException,
-            'Must specify either a user or a group',
-        ):
-            self.cloud.revoke_role(self.role_data.role_name)
-        self.assert_calls()
-
-    def test_grant_no_user_or_group(self):
-        uris = self.__get(
-            'role', self.role_data, 'role_name', [], use_name=True
-        )
-        uris.extend(
-            self.__user_mocks(self.user_data, use_name=True, is_found=False)
-        )
-        self.register_uris(uris)
-
-        with testtools.ExpectedException(
-            exceptions.SDKException,
-            'Must specify either a user or a group',
-        ):
-            self.cloud.grant_role(
-                self.role_data.role_name, user=self.user_data.name
-            )
-        self.assert_calls()
-
-    def test_revoke_no_user_or_group(self):
-        uris = self.__get(
-            'role', self.role_data, 'role_name', [], use_name=True
-        )
-        uris.extend(
-            self.__user_mocks(self.user_data, use_name=True, is_found=False)
-        )
-        self.register_uris(uris)
-
         with testtools.ExpectedException(
             exceptions.SDKException,
             'Must specify either a user or a group',
         ):
             self.cloud.revoke_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1853,12 +1931,6 @@ class TestRoleAssignment(base.TestCase):
         uris = self.__get(
             'role', self.role_data, 'role_name', [], use_name=True
         )
-        uris.extend(self.__user_mocks(self.user_data, use_name=True))
-        uris.extend(
-            self.__get(
-                'group', self.group_data, 'group_name', [], use_name=True
-            )
-        )
         self.register_uris(uris)
 
         with testtools.ExpectedException(
@@ -1869,6 +1941,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 group=self.group_data.group_name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -1892,6 +1965,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 group=self.group_data.group_name,
+                inherited=self.IS_INHERITED,
             )
 
     def test_grant_both_project_and_domain(self):
@@ -1919,6 +1993,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=404,
@@ -1934,6 +2009,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -1947,6 +2023,7 @@ class TestRoleAssignment(base.TestCase):
                 user=self.user_data.name,
                 project=self.project_data.project_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         )
 
@@ -1975,6 +2052,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     complete_qs=True,
                     status_code=204,
@@ -1990,6 +2068,7 @@ class TestRoleAssignment(base.TestCase):
                             'roles',
                             self.role_data.role_id,
                         ],
+                        inherited=self.IS_INHERITED,
                     ),
                     status_code=200,
                 ),
@@ -2003,15 +2082,14 @@ class TestRoleAssignment(base.TestCase):
                 user=self.user_data.name,
                 project=self.project_data.project_name,
                 domain=self.domain_data.domain_name,
+                inherited=self.IS_INHERITED,
             )
         )
 
     def test_grant_no_project_or_domain(self):
         uris = self._get_mock_role_query_urls(
             self.role_data,
-            user_data=self.user_data,
             use_role_name=True,
-            use_user_name=True,
         )
 
         self.register_uris(uris)
@@ -2021,16 +2099,16 @@ class TestRoleAssignment(base.TestCase):
             'Must specify either a domain, project or system',
         ):
             self.cloud.grant_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                user=self.user_data.name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
     def test_revoke_no_project_or_domain_or_system(self):
         uris = self._get_mock_role_query_urls(
             self.role_data,
-            user_data=self.user_data,
             use_role_name=True,
-            use_user_name=True,
         )
 
         self.register_uris(uris)
@@ -2040,7 +2118,9 @@ class TestRoleAssignment(base.TestCase):
             'Must specify either a domain, project or system',
         ):
             self.cloud.revoke_role(
-                self.role_data.role_name, user=self.user_data.name
+                self.role_data.role_name,
+                user=self.user_data.name,
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -2068,6 +2148,7 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain='baddomain',
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
 
@@ -2095,5 +2176,10 @@ class TestRoleAssignment(base.TestCase):
                 self.role_data.role_name,
                 user=self.user_data.name,
                 domain='baddomain',
+                inherited=self.IS_INHERITED,
             )
         self.assert_calls()
+
+
+class TestInheritedRoleAssignment(TestRoleAssignment):
+    IS_INHERITED = True
