@@ -26,7 +26,7 @@ from openstack.tests.unit import base
 
 class TestUpdateServer(base.TestCase):
     def setUp(self):
-        super(TestUpdateServer, self).setUp()
+        super().setUp()
         self.server_id = str(uuid.uuid4())
         self.server_name = self.getUniqueString('name')
         self.updated_server_name = self.getUniqueString('name2')
@@ -57,7 +57,7 @@ class TestUpdateServer(base.TestCase):
                         'compute',
                         'public',
                         append=['servers', 'detail'],
-                        qs_elements=['name=%s' % self.server_name],
+                        qs_elements=[f'name={self.server_name}'],
                     ),
                     json={'servers': [self.fake_server]},
                 ),
@@ -108,7 +108,7 @@ class TestUpdateServer(base.TestCase):
                         'compute',
                         'public',
                         append=['servers', 'detail'],
-                        qs_elements=['name=%s' % self.server_name],
+                        qs_elements=[f'name={self.server_name}'],
                     ),
                     json={'servers': [self.fake_server]},
                 ),
