@@ -12,6 +12,7 @@
 
 import contextlib
 import datetime
+import fixtures
 from unittest import mock
 import uuid
 import warnings
@@ -20,12 +21,14 @@ from openstack.block_storage.v3 import volume
 from openstack.compute.v2 import _proxy
 from openstack.compute.v2 import aggregate
 from openstack.compute.v2 import availability_zone as az
+from openstack.compute.v2 import console_auth_token
 from openstack.compute.v2 import extension
 from openstack.compute.v2 import flavor
 from openstack.compute.v2 import hypervisor
 from openstack.compute.v2 import image
 from openstack.compute.v2 import keypair
 from openstack.compute.v2 import migration
+from openstack.compute.v2 import quota_class_set
 from openstack.compute.v2 import quota_set
 from openstack.compute.v2 import server
 from openstack.compute.v2 import server_action
@@ -37,14 +40,17 @@ from openstack.compute.v2 import server_remote_console
 from openstack.compute.v2 import service
 from openstack.compute.v2 import usage
 from openstack.compute.v2 import volume_attachment
-from openstack import resource
+from openstack.identity.v3 import project
+from openstack import proxy as proxy_base
+from openstack.tests.unit import base
 from openstack.tests.unit import test_proxy_base
+from openstack import types
 from openstack import warnings as os_warnings
 
 
 class TestComputeProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestComputeProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -264,18 +270,32 @@ class TestKeyPair(TestComputeProxy):
         self.verify_create(self.proxy.create_keypair, keypair.Keypair)
 
     def test_keypair_delete(self):
-        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, False)
+        self._verify(
+            "openstack.compute.v2.keypair.Keypair.delete",
+            self.proxy.delete_keypair,
+            method_args=["value"],
+            expected_args=[self.proxy],
+            expected_kwargs={"params": {}},
+        )
 
     def test_keypair_delete_ignore(self):
-        self.verify_delete(self.proxy.delete_keypair, keypair.Keypair, True)
+        self._verify(
+            "openstack.compute.v2.keypair.Keypair.delete",
+            self.proxy.delete_keypair,
+            method_args=["value", True],
+            method_kwargs={"user_id": "fake_user"},
+            expected_args=[self.proxy],
+            expected_kwargs={"params": {"user_id": "fake_user"}},
+        )
 
     def test_keypair_delete_user_id(self):
-        self.verify_delete(
+        self._verify(
+            "openstack.compute.v2.keypair.Keypair.delete",
             self.proxy.delete_keypair,
-            keypair.Keypair,
-            True,
-            method_kwargs={'user_id': 'fake_user'},
-            expected_kwargs={'user_id': 'fake_user'},
+            method_args=["value"],
+            method_kwargs={"user_id": "fake_user"},
+            expected_args=[self.proxy],
+            expected_kwargs={"params": {"user_id": "fake_user"}},
         )
 
     def test_keypair_find(self):
@@ -290,14 +310,28 @@ class TestKeyPair(TestComputeProxy):
         )
 
     def test_keypair_get(self):
-        self.verify_get(self.proxy.get_keypair, keypair.Keypair)
+        self._verify(
+            "openstack.compute.v2.keypair.Keypair.fetch",
+            self.proxy.get_keypair,
+            method_args=["value"],
+            method_kwargs={},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                "error_message": "No Keypair found for value",
+            },
+        )
 
     def test_keypair_get_user_id(self):
-        self.verify_get(
+        self._verify(
+            "openstack.compute.v2.keypair.Keypair.fetch",
             self.proxy.get_keypair,
-            keypair.Keypair,
-            method_kwargs={'user_id': 'fake_user'},
-            expected_kwargs={'user_id': 'fake_user'},
+            method_args=["value"],
+            method_kwargs={"user_id": "fake_user"},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                "error_message": "No Keypair found for value",
+                "user_id": "fake_user",
+            },
         )
 
     def test_keypairs(self):
@@ -312,6 +346,68 @@ class TestKeyPair(TestComputeProxy):
         )
 
 
+class TestKeyPairUrl(base.TestCase):
+    def setUp(self):
+        super().setUp()
+        self.useFixture(
+            fixtures.MonkeyPatch(
+                "openstack.utils.maximum_supported_microversion",
+                lambda *args, **kwargs: "2.10",
+            )
+        )
+
+    def test_keypair_find_user_id(self):
+        self.register_uris(
+            [
+                dict(
+                    method="GET",
+                    uri=self.get_mock_url(
+                        "compute",
+                        "public",
+                        append=["os-keypairs", "fake_keypair"],
+                        qs_elements=["user_id=fake_user"],
+                    ),
+                ),
+            ]
+        )
+
+        self.cloud.compute.find_keypair("fake_keypair", user_id="fake_user")
+
+    def test_keypair_get_user_id(self):
+        self.register_uris(
+            [
+                dict(
+                    method="GET",
+                    uri=self.get_mock_url(
+                        "compute",
+                        "public",
+                        append=["os-keypairs", "fake_keypair"],
+                        qs_elements=["user_id=fake_user"],
+                    ),
+                ),
+            ]
+        )
+
+        self.cloud.compute.get_keypair("fake_keypair", user_id="fake_user")
+
+    def test_keypair_delete_user_id(self):
+        self.register_uris(
+            [
+                dict(
+                    method="DELETE",
+                    uri=self.get_mock_url(
+                        "compute",
+                        "public",
+                        append=["os-keypairs", "fake_keypair"],
+                        qs_elements=["user_id=fake_user"],
+                    ),
+                ),
+            ]
+        )
+
+        self.cloud.compute.delete_keypair("fake_keypair", user_id="fake_user")
+
+
 class TestAggregate(TestComputeProxy):
     def test_aggregate_create(self):
         self.verify_create(self.proxy.create_aggregate, aggregate.Aggregate)
@@ -525,7 +621,7 @@ class TestVolumeAttachment(TestComputeProxy):
 
             self.assertEqual(1, len(w))
             self.assertEqual(
-                os_warnings.OpenStackDeprecationWarning,
+                os_warnings.RemovedInSDK50Warning,
                 w[-1].category,
             )
             self.assertIn(
@@ -550,6 +646,7 @@ class TestVolumeAttachment(TestComputeProxy):
             volume_attachment.VolumeAttachment,
             method_args=[],
             method_kwargs={'server': 'server_id', 'volume': 'volume_id'},
+            expected_args=[None],
             expected_kwargs={
                 'id': 'volume_id',
                 'server_id': 'server_id',
@@ -569,7 +666,7 @@ class TestVolumeAttachment(TestComputeProxy):
             ignore_missing=False,
             method_args=[fake_server, fake_volume],
             method_kwargs={},
-            expected_args=[],
+            expected_args=[None],
             expected_kwargs={
                 'id': fake_volume.id,
                 'server_id': fake_server.id,
@@ -588,7 +685,7 @@ class TestVolumeAttachment(TestComputeProxy):
             ignore_missing=True,
             method_args=[fake_server, fake_volume],
             method_kwargs={},
-            expected_args=[],
+            expected_args=[None],
             expected_kwargs={
                 'id': fake_volume.id,
                 'server_id': fake_server.id,
@@ -614,7 +711,7 @@ class TestVolumeAttachment(TestComputeProxy):
                 ignore_missing=False,
                 method_args=[fake_volume.id, fake_server.id],
                 method_kwargs={},
-                expected_args=[],
+                expected_args=[None],
                 expected_kwargs={
                     'id': fake_volume.id,
                     'server_id': fake_server.id,
@@ -961,6 +1058,30 @@ class TestCompute(TestComputeProxy):
     def test_server_update(self):
         self.verify_update(self.proxy.update_server, server.Server)
 
+    def test_server_change_password(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.change_password",
+            self.proxy.change_server_password,
+            method_args=["value", "password"],
+            expected_args=[self.proxy, "password"],
+        )
+
+    def test_server_get_password(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.get_password",
+            self.proxy.get_server_password,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
+    def test_server_clear_password(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.clear_password",
+            self.proxy.clear_server_password,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
     def test_server_wait_for(self):
         value = server.Server(id='1234')
         self.verify_wait_for_status(
@@ -1181,7 +1302,12 @@ class TestCompute(TestComputeProxy):
             self.proxy.evacuate_server,
             method_args=["value"],
             expected_args=[self.proxy],
-            expected_kwargs={"host": None, "admin_pass": None, "force": None},
+            expected_kwargs={
+                "host": None,
+                "admin_pass": None,
+                "force": None,
+                "on_shared_storage": None,
+            },
         )
 
     def test_server_evacuate_with_options(self):
@@ -1189,11 +1315,13 @@ class TestCompute(TestComputeProxy):
             "openstack.compute.v2.server.Server.evacuate",
             self.proxy.evacuate_server,
             method_args=["value", 'HOST2', 'NEW_PASS', True],
+            method_kwargs={'on_shared_storage': False},
             expected_args=[self.proxy],
             expected_kwargs={
                 "host": "HOST2",
                 "admin_pass": 'NEW_PASS',
                 "force": True,
+                "on_shared_storage": False,
             },
         )
 
@@ -1243,6 +1371,20 @@ class TestCompute(TestComputeProxy):
             self.proxy.unshelve_server,
             method_args=["value"],
             expected_args=[self.proxy],
+            expected_kwargs={
+                "host": None,
+                "availability_zone": types.UNSET,
+            },
+        )
+
+    def test_server_unshelve_with_options(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.unshelve",
+            self.proxy.unshelve_server,
+            method_args=["value"],
+            method_kwargs={"host": "HOST2", "availability_zone": "AZ2"},
+            expected_args=[self.proxy],
+            expected_kwargs={"host": "HOST2", "availability_zone": "AZ2"},
         )
 
     def test_server_trigger_dump(self):
@@ -1253,6 +1395,30 @@ class TestCompute(TestComputeProxy):
             expected_args=[self.proxy],
         )
 
+    def test_server_add_tag(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.add_tag",
+            self.proxy.add_tag_to_server,
+            method_args=["value", "tag"],
+            expected_args=[self.proxy, "tag"],
+        )
+
+    def test_server_remove_tag(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.remove_tag",
+            self.proxy.remove_tag_from_server,
+            method_args=["value", "tag"],
+            expected_args=[self.proxy, "tag"],
+        )
+
+    def test_server_remove_tags(self):
+        self._verify(
+            "openstack.compute.v2.server.Server.remove_all_tags",
+            self.proxy.remove_tags_from_server,
+            method_args=["value"],
+            expected_args=[self.proxy],
+        )
+
     def test_get_server_output(self):
         self._verify(
             "openstack.compute.v2.server.Server.get_console_output",
@@ -1375,9 +1541,15 @@ class TestCompute(TestComputeProxy):
         self._verify(
             'openstack.compute.v2.server.Server.live_migrate',
             self.proxy.live_migrate_server,
-            method_args=["value", "host1", False],
-            expected_args=[self.proxy, "host1"],
-            expected_kwargs={'force': False, 'block_migration': None},
+            method_args=["value"],
+            method_kwargs={'host': 'host1', 'force': False},
+            expected_args=[self.proxy],
+            expected_kwargs={
+                'host': 'host1',
+                'force': False,
+                'block_migration': None,
+                'disk_over_commit': None,
+            },
         )
 
     def test_abort_server_migration(self):
@@ -1450,7 +1622,7 @@ class TestCompute(TestComputeProxy):
         self._verify(
             'openstack.compute.v2.server.Server.add_security_group',
             self.proxy.add_security_group_to_server,
-            method_args=["value", {'id': 'id', 'name': 'sg'}],
+            method_args=["value", 'sg'],
             expected_args=[self.proxy, 'sg'],
         )
 
@@ -1458,7 +1630,7 @@ class TestCompute(TestComputeProxy):
         self._verify(
             'openstack.compute.v2.server.Server.remove_security_group',
             self.proxy.remove_security_group_from_server,
-            method_args=["value", {'id': 'id', 'name': 'sg'}],
+            method_args=["value", 'sg'],
             expected_args=[self.proxy, 'sg'],
         )
 
@@ -1466,7 +1638,7 @@ class TestCompute(TestComputeProxy):
         self.verify_list(self.proxy.usages, usage.Usage)
 
     def test_usages__with_kwargs(self):
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc).replace(tzinfo=None)
         start = now - datetime.timedelta(weeks=4)
         end = end = now + datetime.timedelta(days=1)
         self.verify_list(
@@ -1490,7 +1662,7 @@ class TestCompute(TestComputeProxy):
         )
 
     def test_get_usage__with_kwargs(self):
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(datetime.timezone.utc).replace(tzinfo=None)
         start = now - datetime.timedelta(weeks=4)
         end = end = now + datetime.timedelta(days=1)
         self._verify(
@@ -1562,51 +1734,80 @@ class TestCompute(TestComputeProxy):
         self.assertEqual(console_fake['url'], ret['url'])
 
 
-class TestQuota(TestComputeProxy):
-    def test_get(self):
+class TestQuotaClassSet(TestComputeProxy):
+    def test_quota_class_set_get(self):
+        self.verify_get(
+            self.proxy.get_quota_class_set, quota_class_set.QuotaClassSet
+        )
+
+    def test_quota_class_set_update(self):
+        self.verify_update(
+            self.proxy.update_quota_class_set,
+            quota_class_set.QuotaClassSet,
+            False,
+        )
+
+
+class TestQuotaSet(TestComputeProxy):
+    def test_quota_set_get(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
             method_args=['prj'],
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                None,
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'usage': False,
+                'microversion': None,
+                'resource_response_key': None,
             },
             method_result=quota_set.QuotaSet(),
             expected_result=quota_set.QuotaSet(),
         )
 
-    def test_get_query(self):
+    def test_quota_set_get_query(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set,
             method_args=['prj'],
             method_kwargs={'usage': True, 'user_id': 'uid'},
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                '/os-quota-sets/%(project_id)s/detail',
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'usage': True,
+                'microversion': None,
+                'resource_response_key': None,
                 'user_id': 'uid',
             },
         )
 
-    def test_get_defaults(self):
+    def test_quota_set_get_defaults(self):
         self._verify(
             'openstack.resource.Resource.fetch',
             self.proxy.get_quota_set_defaults,
             method_args=['prj'],
-            expected_args=[self.proxy],
+            expected_args=[
+                self.proxy,
+                False,
+                '/os-quota-sets/%(project_id)s/defaults',
+                None,
+                False,
+            ],
             expected_kwargs={
-                'error_message': None,
-                'requires_id': False,
-                'base_path': '/os-quota-sets/defaults',
+                'microversion': None,
+                'resource_response_key': None,
             },
         )
 
-    def test_reset(self):
+    def test_quota_set_reset(self):
         self._verify(
             'openstack.resource.Resource.delete',
             self.proxy.revert_quota_set,
@@ -1616,22 +1817,52 @@ class TestQuota(TestComputeProxy):
             expected_kwargs={'user_id': 'uid'},
         )
 
-    @mock.patch('openstack.proxy.Proxy._get_resource', autospec=True)
-    def test_update(self, gr_mock):
-        gr_mock.return_value = resource.Resource()
-        gr_mock.commit = mock.Mock()
+    @mock.patch.object(proxy_base.Proxy, "_get_resource")
+    def test_quota_set_update(self, mock_get):
+        fake_project = project.Project(id='prj')
+        fake_quota_set = quota_set.QuotaSet(project_id='prj')
+        mock_get.side_effect = [fake_project, fake_quota_set]
+
         self._verify(
             'openstack.resource.Resource.commit',
             self.proxy.update_quota_set,
-            method_args=['qs'],
-            method_kwargs={
-                'query': {'user_id': 'uid'},
-                'a': 'b',
-            },
+            method_args=['prj'],
+            method_kwargs={'ram': 123},
             expected_args=[self.proxy],
-            expected_kwargs={'user_id': 'uid'},
+            expected_kwargs={},
         )
-        gr_mock.assert_called_with(self.proxy, quota_set.QuotaSet, 'qs', a='b')
+        mock_get.assert_has_calls(
+            [
+                mock.call(project.Project, 'prj'),
+                mock.call(quota_set.QuotaSet, None, project_id='prj', ram=123),
+            ]
+        )
+
+    @mock.patch.object(proxy_base.Proxy, "_get_resource")
+    def test_quota_set_update__legacy(self, mock_get):
+        fake_quota_set = quota_set.QuotaSet(project_id='prj')
+        mock_get.side_effect = [fake_quota_set]
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+
+            self._verify(
+                'openstack.resource.Resource.commit',
+                self.proxy.update_quota_set,
+                method_args=[fake_quota_set],
+                method_kwargs={'ram': 123},
+                expected_args=[self.proxy],
+                expected_kwargs={},
+            )
+
+            self.assertEqual(1, len(w))
+            self.assertEqual(
+                os_warnings.RemovedInSDK50Warning,
+                w[-1].category,
+            )
+            self.assertIn(
+                "The signature of 'update_quota_set' has changed ",
+                str(w[-1]),
+            )
 
 
 class TestServerAction(TestComputeProxy):
@@ -1655,3 +1886,11 @@ class TestServerAction(TestComputeProxy):
             method_kwargs={'server': 'server_a'},
             expected_kwargs={'server_id': 'server_a'},
         )
+
+
+class TestValidateConsoleAuthToken(TestComputeProxy):
+    def test_validate_console_auth_token(self):
+        self.verify_get(
+            self.proxy.validate_console_auth_token,
+            console_auth_token.ConsoleAuthToken,
+        )
