@@ -31,7 +31,7 @@ from openstack.tests.unit import test_proxy_base
 
 class TestOrchestrationProxy(test_proxy_base.TestProxyBase):
     def setUp(self):
-        super(TestOrchestrationProxy, self).setUp()
+        super().setUp()
         self.proxy = _proxy.Proxy(self.session)
 
 
@@ -96,7 +96,7 @@ class TestOrchestrationStack(TestOrchestrationProxy):
 
     def test_update_stack(self):
         self._verify(
-            'openstack.orchestration.v1.stack.Stack.update',
+            'openstack.orchestration.v1.stack.Stack.commit',
             self.proxy.update_stack,
             expected_result='result',
             method_args=['stack'],
@@ -106,7 +106,7 @@ class TestOrchestrationStack(TestOrchestrationProxy):
 
     def test_update_stack_preview(self):
         self._verify(
-            'openstack.orchestration.v1.stack.Stack.update',
+            'openstack.orchestration.v1.stack.Stack.commit',
             self.proxy.update_stack,
             expected_result='result',
             method_args=['stack'],
@@ -348,12 +348,12 @@ class TestOrchestrationResource(TestOrchestrationProxy):
     @mock.patch.object(resource.Resource, 'list')
     def test_resources_stack_not_found(self, mock_list, mock_find):
         stack_name = 'test_stack'
-        mock_find.side_effect = exceptions.ResourceNotFound(
+        mock_find.side_effect = exceptions.NotFoundException(
             'No stack found for test_stack'
         )
 
         ex = self.assertRaises(
-            exceptions.ResourceNotFound, self.proxy.resources, stack_name
+            exceptions.NotFoundException, self.proxy.resources, stack_name
         )
         self.assertEqual('No stack found for test_stack', str(ex))
 
