@@ -21,6 +21,7 @@ from openstack import exceptions
 from openstack import proxy
 from openstack import resource
 from openstack.tests.unit import base
+from openstack.tests.unit import fakes
 from openstack import utils
 
 
@@ -73,48 +74,6 @@ class TestProxyPrivate(base.TestCase):
         self.fake_proxy = proxy.Proxy(self.session)
         self.fake_proxy._connection = self.cloud
 
-    def _test_correct(self, value):
-        decorated = proxy._check_resource(strict=False)(self.sot.method)
-        rv = decorated(self.sot, resource.Resource, value)
-
-        self.assertEqual(value, rv)
-
-    def test__check_resource_correct_resource(self):
-        res = resource.Resource()
-        self._test_correct(res)
-
-    def test__check_resource_notstrict_id(self):
-        self._test_correct("abc123-id")
-
-    def test__check_resource_strict_id(self):
-        decorated = proxy._check_resource(strict=True)(self.sot.method)
-        self.assertRaisesRegex(
-            ValueError,
-            "A Resource must be passed",
-            decorated,
-            self.sot,
-            resource.Resource,
-            "this-is-not-a-resource",
-        )
-
-    def test__check_resource_incorrect_resource(self):
-        class OneType(resource.Resource):
-            pass
-
-        class AnotherType(resource.Resource):
-            pass
-
-        value = AnotherType()
-        decorated = proxy._check_resource(strict=False)(self.sot.method)
-        self.assertRaisesRegex(
-            ValueError,
-            "Expected OneType but received AnotherType",
-            decorated,
-            self.sot,
-            OneType,
-            value,
-        )
-
     def test__get_uri_attribute_no_parent(self):
         class Child(resource.Resource):
             something = resource.Body("something")
@@ -237,7 +196,8 @@ class TestProxyDelete(base.TestCase):
 
     def test_delete_ignore_missing(self):
         self.res.delete.side_effect = exceptions.NotFoundException(
-            message="test", http_status=404
+            message="test",
+            response=fakes.FakeResponse(status_code=404, data={'error': None}),
         )
 
         rv = self.sot._delete(DeleteableResource, self.fake_id)
@@ -245,7 +205,8 @@ class TestProxyDelete(base.TestCase):
 
     def test_delete_NotFound(self):
         self.res.delete.side_effect = exceptions.NotFoundException(
-            message="test", http_status=404
+            message="test",
+            response=fakes.FakeResponse(status_code=404, data={'error': None}),
         )
 
         self.assertRaisesRegex(
@@ -259,8 +220,9 @@ class TestProxyDelete(base.TestCase):
         )
 
     def test_delete_HttpException(self):
-        self.res.delete.side_effect = exceptions.HttpException(
-            message="test", http_status=500
+        self.res.delete.side_effect = exceptions.ResourceNotFound(
+            message="test",
+            response=fakes.FakeResponse(status_code=500, data={'error': None}),
         )
 
         self.assertRaises(
@@ -468,7 +430,8 @@ class TestProxyGet(base.TestCase):
 
     def test_get_not_found(self):
         self.res.fetch.side_effect = exceptions.NotFoundException(
-            message="test", http_status=404
+            message="test",
+            response=fakes.FakeResponse(status_code=404, data={'error': None}),
         )
 
         self.assertRaisesRegex(
